@@ -65,11 +65,12 @@ typedef gboolean (*GsAppListFilterFunc)		(GsApp		*app,
 						 gpointer	 user_data);
 
 GsAppList	*gs_app_list_new		(void);
+GsAppList	*gs_app_list_copy		(GsAppList	*list);
 void		 gs_app_list_add		(GsAppList	*list,
 						 GsApp		*app);
 void		 gs_app_list_add_list		(GsAppList	*list,
 						 GsAppList	*donor);
-void		 gs_app_list_remove		(GsAppList	*list,
+gboolean	 gs_app_list_remove		(GsAppList	*list,
 						 GsApp		*app);
 GsApp		*gs_app_list_index		(GsAppList	*list,
 						 guint		 idx);
@@ -82,5 +83,7 @@ void		 gs_app_list_sort		(GsAppList	*list,
 void		 gs_app_list_filter		(GsAppList	*list,
 						 GsAppListFilterFunc func,
 						 gpointer	 user_data);
+void		 gs_app_list_override_progress	(GsAppList	*list,
+						 guint		 progress);
 
 G_END_DECLS
