@@ -29,15 +29,27 @@ gboolean	 gs_appstream_search			(GsPlugin	*plugin,
 							 GsAppList	*list,
 							 GCancellable	*cancellable,
 							 GError		**error);
-gboolean	 gs_appstream_add_categories		(XbSilo		*silo,
+gboolean	 gs_appstream_search_developer_apps	(GsPlugin	*plugin,
+							 XbSilo		*silo,
+							 const gchar * const *values,
+							 GsAppList	*list,
+							 GCancellable	*cancellable,
+							 GError		**error);
+gboolean	 gs_appstream_refine_category_sizes	(XbSilo		*silo,
 							 GPtrArray	*list,
 							 GCancellable	*cancellable,
 							 GError		**error);
-gboolean	 gs_appstream_add_category_apps		(XbSilo		*silo,
+gboolean	 gs_appstream_add_category_apps		(GsPlugin	*plugin,
+							 XbSilo		*silo,
 							 GsCategory	*category,
 							 GsAppList	*list,
 							 GCancellable	*cancellable,
 							 GError		**error);
+gboolean	 gs_appstream_add_installed		(GsPlugin	*plugin,
+							 XbSilo		*silo,
+							 GsAppList	*list,
+							 GCancellable	*cancellable,
+							 GError		**error);
 gboolean	 gs_appstream_add_popular		(XbSilo		*silo,
 							 GsAppList	*list,
 							 GCancellable	*cancellable,
@@ -46,6 +58,11 @@ gboolean	 gs_appstream_add_featured		(XbSilo		*silo,
 							 GsAppList	*list,
 							 GCancellable	*cancellable,
 							 GError		**error);
+gboolean	 gs_appstream_add_deployment_featured	(XbSilo		*silo,
+							 const gchar * const *deployments,
+							 GsAppList	*list,
+							 GCancellable	*cancellable,
+							 GError		**error);
 gboolean	 gs_appstream_add_alternates		(XbSilo		*silo,
 							 GsApp		*app,
 							 GsAppList	*list,
