@@ -48,8 +48,10 @@
  * Since: 42
  */
 
+#include <errno.h>
 #include <glib.h>
 #include <glib/gi18n.h>
+#include <glib/gstdio.h>
 #include <libsoup/soup.h>
 
 #include "gs-external-appstream-utils.h"
@@ -66,6 +68,15 @@ gs_external_appstream_utils_get_file_cache_path (const gchar *file_name)
 	return g_build_filename (APPSTREAM_SYSTEM_DIR, prefixed_file_name, NULL);
 }
 
+/* To be able to delete old files, when the path changed */
+gchar *
+gs_external_appstream_utils_get_legacy_file_cache_path (const gchar *file_name)
+{
+	g_autofree gchar *prefixed_file_name = g_strdup_printf (EXTERNAL_APPSTREAM_PREFIX "-%s",
+								file_name);
+	return g_build_filename (LOCALSTATEDIR "/cache/app-info/xmls", prefixed_file_name, NULL);
+}
+
 const gchar *
 gs_external_appstream_utils_get_system_dir (void)
 {
@@ -99,12 +110,12 @@ gs_external_appstream_install (const gchar   *appstream_file,
 	return g_subprocess_wait_check (subprocess, cancellable, error);
 }
 
-static void download_system_cb (GObject      *source_object,
+static void download_replace_file_cb (GObject      *source_object,
+				      GAsyncResult *result,
+				      gpointer      user_data);
+static void download_stream_cb (GObject      *source_object,
                                 GAsyncResult *result,
                                 gpointer      user_data);
-static void download_user_cb (GObject      *source_object,
-                              GAsyncResult *result,
-                              gpointer      user_data);
 
 /* A tuple to store the last-received progress data for a single download.
  * Each download (refresh_url_async()) has a pointer to the relevant
@@ -116,6 +127,34 @@ typedef struct {
 	gsize total_download_size;
 } ProgressTuple;
 
+typedef struct {
+	/* Input data. */
+	gchar *url;  /* (not nullable) (owned) */
+	GTask *task;  /* (not nullable) (owned) */
+	GFile *output_file;  /* (not nullable) (owned) */
+	ProgressTuple *progress_tuple;  /* (not nullable) */
+	SoupSession *soup_session;  /* (not nullable) (owned) */
+	gboolean system_wide;
+
+	/* In-progress data. */
+	gchar *last_etag;  /* (nullable) (owned) */
+	GDateTime *last_modified_date;  /* (nullable) (owned) */
+} DownloadAppStreamData;
+
+static void
+download_appstream_data_free (DownloadAppStreamData *data)
+{
+	g_free (data->url);
+	g_clear_object (&data->task);
+	g_clear_object (&data->output_file);
+	g_clear_object (&data->soup_session);
+	g_free (data->last_etag);
+	g_clear_pointer (&data->last_modified_date, g_date_time_unref);
+	g_free (data);
+}
+
+G_DEFINE_AUTOPTR_CLEANUP_FUNC (DownloadAppStreamData, download_appstream_data_free)
+
 static void
 refresh_url_progress_cb (gsize    bytes_downloaded,
                          gsize    total_download_size,
@@ -147,9 +186,11 @@ refresh_url_async (GSettings           *settings,
 	g_autofree gchar *hash = NULL;
 	g_autofree gchar *target_file_path = NULL;
 	g_autoptr(GFile) target_file = NULL;
+	g_autoptr(GFile) tmp_file_parent = NULL;
 	g_autoptr(GFile) tmp_file = NULL;
 	g_autoptr(GsApp) app_dl = gs_app_new ("external-appstream");
 	g_autoptr(GError) local_error = NULL;
+	DownloadAppStreamData *data;
 	gboolean system_wide;
 
 	task = g_task_new (NULL, cancellable, callback, user_data);
@@ -170,15 +211,32 @@ refresh_url_async (GSettings           *settings,
 	system_wide = g_settings_get_boolean (settings, "external-appstream-system-wide");
 
 	/* Check cache file age. */
-	if (system_wide)
+	if (system_wide) {
 		target_file_path = gs_external_appstream_utils_get_file_cache_path (basename);
-	else
+	} else {
+		g_autofree gchar *legacy_file_path = NULL;
+
 		target_file_path = g_build_filename (g_get_user_data_dir (),
 						     "swcatalog",
 						     "xml",
 						     basename,
 						     NULL);
 
+		/* Delete an old file, from a legacy location */
+		legacy_file_path = g_build_filename (g_get_user_data_dir (),
+						     "app-info",
+						     "xmls",
+						     basename,
+						     NULL);
+
+		if (g_unlink (legacy_file_path) == -1) {
+			int errn = errno;
+			if (errn != ENOENT)
+				g_debug ("Failed to unlink '%s': %s", legacy_file_path, g_strerror (errn));
+
+		}
+	}
+
 	target_file = g_file_new_for_path (target_file_path);
 
 	if (!gs_external_appstream_check (target_file, cache_age_secs)) {
@@ -209,90 +267,137 @@ refresh_url_async (GSettings           *settings,
 		tmp_file = g_object_ref (target_file);
 	}
 
-	g_task_set_task_data (task, g_object_ref (tmp_file), g_object_unref);
-
 	gs_app_set_summary_missing (app_dl,
 				    /* TRANSLATORS: status text when downloading */
 				    _("Downloading extra metadata files…"));
 
-	/* Do the download. */
-	gs_download_file_async (soup_session, url, tmp_file, G_PRIORITY_LOW,
-				refresh_url_progress_cb,
-				progress_tuple,
-				cancellable,
-				system_wide ? download_system_cb : download_user_cb,
-				g_steal_pointer (&task));
+	data = g_new0 (DownloadAppStreamData, 1);
+	data->url = g_strdup (url);
+	data->task = g_object_ref (task);
+	data->output_file = g_object_ref (tmp_file);
+	data->progress_tuple = progress_tuple;
+	data->soup_session = g_object_ref (soup_session);
+	data->system_wide = system_wide;
+	g_task_set_task_data (task, data, (GDestroyNotify) download_appstream_data_free);
+
+	/* Create the destination file’s directory.
+	 * FIXME: This should be made async; it hasn’t done for now as it’s
+	 * likely to be fast. */
+	tmp_file_parent = g_file_get_parent (tmp_file);
+
+	if (tmp_file_parent != NULL &&
+	    !g_file_make_directory_with_parents (tmp_file_parent, cancellable, &local_error) &&
+	    !g_error_matches (local_error, G_IO_ERROR, G_IO_ERROR_EXISTS)) {
+		g_task_return_error (task, g_steal_pointer (&local_error));
+		return;
+	}
+
+	g_clear_error (&local_error);
+
+	/* Query the ETag and modification date of the target file, if the file already exists. For
+	 * system-wide installations, this is the ETag of the AppStream file installed system-wide.
+	 * For local installations, this is just the local output file. */
+	data->last_etag = gs_utils_get_file_etag (target_file, &data->last_modified_date, cancellable);
+	g_debug ("Queried ETag of file %s: %s", g_file_peek_path (target_file), data->last_etag);
+
+	/* Create the output file */
+	g_file_replace_async (tmp_file,
+			      NULL,  /* ETag */
+			      FALSE,  /* make_backup */
+			      G_FILE_CREATE_PRIVATE | G_FILE_CREATE_REPLACE_DESTINATION,
+			      G_PRIORITY_LOW,
+			      cancellable,
+			      download_replace_file_cb,
+			      g_steal_pointer (&task));
 }
 
 static void
-download_system_cb (GObject      *source_object,
+download_replace_file_cb (GObject      *source_object,
+                          GAsyncResult *result,
+                          gpointer      user_data)
+{
+	GFile *output_file = G_FILE (source_object);
+	g_autoptr(GTask) task = g_steal_pointer (&user_data);
+	GCancellable *cancellable = g_task_get_cancellable (task);
+	DownloadAppStreamData *data = g_task_get_task_data (task);
+	g_autoptr(GFileOutputStream) output_stream = NULL;
+	g_autoptr(GError) local_error = NULL;
+
+	output_stream = g_file_replace_finish (output_file, result, &local_error);
+
+	if (output_stream == NULL) {
+		g_task_return_error (task, g_steal_pointer (&local_error));
+		return;
+	}
+
+	/* Do the download. */
+	gs_download_stream_async (data->soup_session,
+				  data->url,
+				  G_OUTPUT_STREAM (output_stream),
+				  data->last_etag,
+				  data->last_modified_date,
+				  G_PRIORITY_LOW,
+				  refresh_url_progress_cb,
+				  data->progress_tuple,
+				  cancellable,
+				  download_stream_cb,
+				  g_steal_pointer (&task));
+}
+
+static void
+download_stream_cb (GObject      *source_object,
                     GAsyncResult *result,
                     gpointer      user_data)
 {
 	SoupSession *soup_session = SOUP_SESSION (source_object);
 	g_autoptr(GTask) task = g_steal_pointer (&user_data);
 	GCancellable *cancellable = g_task_get_cancellable (task);
-	GFile *tmp_file = g_task_get_task_data (task);
+	DownloadAppStreamData *data = g_task_get_task_data (task);
 	g_autoptr(GError) local_error = NULL;
+	g_autofree gchar *new_etag = NULL;
 
-	if (!gs_download_file_finish (soup_session, result, &local_error)) {
-		if (!g_network_monitor_get_network_available (g_network_monitor_get_default ()))
+	if (!gs_download_stream_finish (soup_session, result, &new_etag, NULL, &local_error)) {
+		if (data->system_wide && g_error_matches (local_error, GS_DOWNLOAD_ERROR, GS_DOWNLOAD_ERROR_NOT_MODIFIED)) {
+			g_debug ("External AppStream file not modified, removing temporary download file %s",
+				 g_file_peek_path (data->output_file));
+
+			/* System-wide installs should delete the empty file created when preparing to
+			 * download the external AppStream file. */
+			g_file_delete_async (data->output_file, G_PRIORITY_LOW, NULL, NULL, NULL);
+			g_task_return_boolean (task, TRUE);
+		} else if (!g_network_monitor_get_network_available (g_network_monitor_get_default ())) {
 			g_task_return_new_error (task,
 						 GS_EXTERNAL_APPSTREAM_ERROR,
 						 GS_EXTERNAL_APPSTREAM_ERROR_NO_NETWORK,
 						 "External AppStream could not be downloaded due to being offline");
-		else
+		} else {
 			g_task_return_new_error (task,
 						 GS_EXTERNAL_APPSTREAM_ERROR,
 						 GS_EXTERNAL_APPSTREAM_ERROR_DOWNLOADING,
 						 "Server returned no data for external AppStream file: %s",
 						 local_error->message);
+		}
 		return;
 	}
 
-	g_debug ("Downloaded appstream file %s", g_file_peek_path (tmp_file));
+	g_debug ("Downloaded appstream file %s", g_file_peek_path (data->output_file));
 
-	/* install file systemwide */
-	if (gs_external_appstream_install (g_file_peek_path (tmp_file),
-					   cancellable,
-					   &local_error)) {
-		g_debug ("Installed appstream file %s", g_file_peek_path (tmp_file));
-		g_task_return_boolean (task, TRUE);
-	} else {
-		g_task_return_new_error (task,
-					 GS_EXTERNAL_APPSTREAM_ERROR,
-					 GS_EXTERNAL_APPSTREAM_ERROR_INSTALLING_ON_SYSTEM,
-					 "Error installing external AppStream file on system: %s", local_error->message);
-	}
-}
+	gs_utils_set_file_etag (data->output_file, new_etag, cancellable);
 
-static void
-download_user_cb (GObject      *source_object,
-                  GAsyncResult *result,
-                  gpointer      user_data)
-{
-	SoupSession *soup_session = SOUP_SESSION (source_object);
-	g_autoptr(GTask) task = g_steal_pointer (&user_data);
-	GFile *tmp_file = g_task_get_task_data (task);
-	g_autoptr(GError) local_error = NULL;
-
-	if (!gs_download_file_finish (soup_session, result, &local_error)) {
-		if (!g_network_monitor_get_network_available (g_network_monitor_get_default ()))
+	if (data->system_wide) {
+		/* install file systemwide */
+		if (!gs_external_appstream_install (g_file_peek_path (data->output_file),
+						    cancellable,
+						    &local_error)) {
 			g_task_return_new_error (task,
 						 GS_EXTERNAL_APPSTREAM_ERROR,
-						 GS_EXTERNAL_APPSTREAM_ERROR_NO_NETWORK,
-						 "External AppStream could not be downloaded due to being offline");
-		else
-			g_task_return_new_error (task,
-						 GS_EXTERNAL_APPSTREAM_ERROR,
-						 GS_EXTERNAL_APPSTREAM_ERROR_DOWNLOADING,
-						 "Server returned no data for external AppStream file: %s",
-						 local_error->message);
-		return;
+						 GS_EXTERNAL_APPSTREAM_ERROR_INSTALLING_ON_SYSTEM,
+						 "Error installing external AppStream file on system: %s", local_error->message);
+			return;
+		}
+		g_debug ("Installed appstream file %s", g_file_peek_path (data->output_file));
 	}
 
-	g_debug ("Downloaded appstream file %s", g_file_peek_path (tmp_file));
-
 	g_task_return_boolean (task, TRUE);
 }
 
