@@ -119,7 +119,7 @@ gs_metered_block_on_download_scheduler (GVariant      *parameters,
 	g_autoptr(MwscScheduleEntry) schedule_entry = NULL;
 	g_autofree gchar *parameters_str = NULL;
 	g_autoptr(GMainContext) context = NULL;
-	g_autoptr(GsMainContextPusher) pusher = NULL;
+	g_autoptr(GMainContextPusher) pusher = NULL;
 
 	g_return_val_if_fail (schedule_entry_handle_out != NULL, FALSE);
 
@@ -131,7 +131,7 @@ gs_metered_block_on_download_scheduler (GVariant      *parameters,
 
 	/* Push the context early so that the #MwscScheduler is created to run within it. */
 	context = g_main_context_new ();
-	pusher = gs_main_context_pusher_new (context);
+	pusher = g_main_context_pusher_new (context);
 
 	/* Wait until the download can be scheduled.
 	 * FIXME: In future, downloads could be split up by app, so they can all
@@ -257,13 +257,13 @@ gs_metered_block_app_on_download_scheduler (GsApp         *app,
 {
 	g_auto(GVariantDict) parameters_dict = G_VARIANT_DICT_INIT (NULL);
 	g_autoptr(GVariant) parameters = NULL;
-	guint64 download_size = gs_app_get_size_download (app);
+	guint64 download_size;
 
 	/* Currently no plugins support resumable downloads. This may change in
 	 * future, in which case this parameter should be refactored. */
 	g_variant_dict_insert (&parameters_dict, "resumable", "b", FALSE);
 
-	if (download_size != 0 && download_size != GS_APP_SIZE_UNKNOWABLE) {
+	if (gs_app_get_size_download (app, &download_size) == GS_SIZE_TYPE_VALID) {
 		g_variant_dict_insert (&parameters_dict, "size-minimum", "t", download_size);
 		g_variant_dict_insert (&parameters_dict, "size-maximum", "t", download_size);
 	}
