@@ -65,43 +65,6 @@ gs_plugin_loader_job_process (GsPluginLoader *plugin_loader,
 	return list;
 }
 
-GPtrArray *
-gs_plugin_loader_job_get_categories (GsPluginLoader *plugin_loader,
-				     GsPluginJob *plugin_job,
-				     GCancellable *cancellable,
-				     GError **error)
-{
-	GsPluginLoaderHelper helper;
-	GPtrArray *catlist;
-
-	/* create temp object */
-	helper.res = NULL;
-	helper.context = g_main_context_new ();
-	helper.loop = g_main_loop_new (helper.context, FALSE);
-
-	g_main_context_push_thread_default (helper.context);
-
-	/* run async method */
-	gs_plugin_loader_job_get_categories_async (plugin_loader,
-						   plugin_job,
-						   cancellable,
-						   _helper_finish_sync,
-						   &helper);
-	g_main_loop_run (helper.loop);
-	catlist = gs_plugin_loader_job_get_categories_finish (plugin_loader,
-	                                                      helper.res,
-	                                                      error);
-
-	g_main_context_pop_thread_default (helper.context);
-
-	g_main_loop_unref (helper.loop);
-	g_main_context_unref (helper.context);
-	if (helper.res != NULL)
-		g_object_unref (helper.res);
-
-	return catlist;
-}
-
 gboolean
 gs_plugin_loader_job_action (GsPluginLoader *plugin_loader,
 			     GsPluginJob *plugin_job,
