@@ -26,10 +26,6 @@ gboolean	 gs_plugin_loader_job_action		(GsPluginLoader	*plugin_loader,
 							 GsPluginJob	*plugin_job,
 							 GCancellable	*cancellable,
 							 GError		**error);
-GPtrArray	*gs_plugin_loader_job_get_categories	(GsPluginLoader	*plugin_loader,
-							 GsPluginJob	*plugin_job,
-							 GCancellable	*cancellable,
-							 GError		**error);
 GsApp		*gs_plugin_loader_app_create		(GsPluginLoader	*plugin_loader,
 							 const gchar	*unique_id,
 							 GCancellable	*cancellable,
@@ -37,5 +33,10 @@ GsApp		*gs_plugin_loader_app_create		(GsPluginLoader	*plugin_loader,
 GsApp		*gs_plugin_loader_get_system_app	(GsPluginLoader	*plugin_loader,
 							 GCancellable	*cancellable,
 							 GError		**error);
+gboolean	 gs_plugin_loader_setup			(GsPluginLoader	*plugin_loader,
+							 const gchar * const *allowlist,
+							 const gchar * const *blocklist,
+							 GCancellable	 *cancellable,
+							 GError		**error);
 
 G_END_DECLS
