@@ -14,16 +14,19 @@
 #include "gs-app.h"
 #include "gs-category.h"
 #include "gs-category-manager.h"
+#include "gs-odrs-provider.h"
 #include "gs-plugin-event.h"
-#include "gs-plugin-private.h"
-#include "gs-plugin-job.h"
+#include "gs-plugin.h"
 
 G_BEGIN_DECLS
 
 #define GS_TYPE_PLUGIN_LOADER		(gs_plugin_loader_get_type ())
 G_DECLARE_FINAL_TYPE (GsPluginLoader, gs_plugin_loader, GS, PLUGIN_LOADER, GObject)
 
-GsPluginLoader	*gs_plugin_loader_new			(void);
+#include "gs-plugin-job.h"
+
+GsPluginLoader	*gs_plugin_loader_new			(GDBusConnection *session_bus_connection,
+							 GDBusConnection *system_bus_connection);
 void		 gs_plugin_loader_job_process_async	(GsPluginLoader	*plugin_loader,
 							 GsPluginJob	*plugin_job,
 							 GCancellable	*cancellable,
@@ -35,19 +38,19 @@ GsAppList	*gs_plugin_loader_job_process_finish	(GsPluginLoader	*plugin_loader,
 gboolean	 gs_plugin_loader_job_action_finish	(GsPluginLoader	*plugin_loader,
 							 GAsyncResult	*res,
 							 GError		**error);
-void		 gs_plugin_loader_job_get_categories_async (GsPluginLoader *plugin_loader,
-							 GsPluginJob	*plugin_job,
+void		 gs_plugin_loader_setup_async		(GsPluginLoader	*plugin_loader,
+							 const gchar * const *allowlist,
+							 const gchar * const *blocklist,
 							 GCancellable	*cancellable,
 							 GAsyncReadyCallback callback,
 							 gpointer	 user_data);
-GPtrArray	*gs_plugin_loader_job_get_categories_finish (GsPluginLoader *plugin_loader,
-							 GAsyncResult	*res,
-							 GError		**error);
-gboolean	 gs_plugin_loader_setup			(GsPluginLoader	*plugin_loader,
-							 gchar		**allowlist,
-							 gchar		**blocklist,
-							 GCancellable	*cancellable,
+gboolean	 gs_plugin_loader_setup_finish		(GsPluginLoader	*plugin_loader,
+							 GAsyncResult	*result,
 							 GError		**error);
+
+void		 gs_plugin_loader_shutdown		(GsPluginLoader	*plugin_loader,
+							 GCancellable	*cancellable);
+
 void		 gs_plugin_loader_dump_state		(GsPluginLoader	*plugin_loader);
 gboolean	 gs_plugin_loader_get_enabled		(GsPluginLoader	*plugin_loader,
 							 const gchar	*plugin_name);
@@ -63,26 +66,38 @@ gboolean	 gs_plugin_loader_get_network_metered	(GsPluginLoader *plugin_loader);
 gboolean	 gs_plugin_loader_get_plugin_supported	(GsPluginLoader	*plugin_loader,
 							 const gchar	*function_name);
 
+GPtrArray	*gs_plugin_loader_get_plugins		(GsPluginLoader	*plugin_loader);
+
 void		 gs_plugin_loader_add_event		(GsPluginLoader *plugin_loader,
 							 GsPluginEvent	*event);
 GPtrArray	*gs_plugin_loader_get_events		(GsPluginLoader	*plugin_loader);
 GsPluginEvent	*gs_plugin_loader_get_event_default	(GsPluginLoader	*plugin_loader);
 void		 gs_plugin_loader_remove_events		(GsPluginLoader	*plugin_loader);
 
-GsApp		*gs_plugin_loader_app_create		(GsPluginLoader	*plugin_loader,
-							 const gchar	*unique_id);
-GsApp		*gs_plugin_loader_get_system_app	(GsPluginLoader	*plugin_loader);
+void		 gs_plugin_loader_app_create_async	(GsPluginLoader	*plugin_loader,
+							 const gchar	*unique_id,
+							 GCancellable	*cancellable,
+							 GAsyncReadyCallback callback,
+							 gpointer	 user_data);
+GsApp		*gs_plugin_loader_app_create_finish	(GsPluginLoader	*plugin_loader,
+							 GAsyncResult	*res,
+							 GError		**error);
+void		 gs_plugin_loader_get_system_app_async	(GsPluginLoader	*plugin_loader,
+							 GCancellable	*cancellable,
+							 GAsyncReadyCallback callback,
+							 gpointer	 user_data);
+GsApp		*gs_plugin_loader_get_system_app_finish	(GsPluginLoader	*plugin_loader,
+							 GAsyncResult	*res,
+							 GError		**error);
+GsOdrsProvider	*gs_plugin_loader_get_odrs_provider	(GsPluginLoader	*plugin_loader);
 
 /* only useful from the self tests */
-void		 gs_plugin_loader_setup_again		(GsPluginLoader	*plugin_loader);
 void		 gs_plugin_loader_clear_caches		(GsPluginLoader	*plugin_loader);
 GsPlugin	*gs_plugin_loader_find_plugin		(GsPluginLoader	*plugin_loader,
 							 const gchar	*plugin_name);
 void            gs_plugin_loader_set_max_parallel_ops  (GsPluginLoader *plugin_loader,
                                                         guint           max_ops);
 
-const gchar	*gs_plugin_loader_get_locale		(GsPluginLoader *plugin_loader);
-
 GsCategoryManager *gs_plugin_loader_get_category_manager (GsPluginLoader *plugin_loader);
 void		 gs_plugin_loader_claim_error		(GsPluginLoader *plugin_loader,
 							 GsPlugin *plugin,
@@ -95,4 +110,13 @@ void		 gs_plugin_loader_claim_job_error	(GsPluginLoader *plugin_loader,
 							 GsPluginJob *job,
 							 const GError *error);
 
+gboolean	 gs_plugin_loader_app_is_valid		(GsApp *app,
+							 GsPluginRefineFlags flags);
+gboolean	 gs_plugin_loader_app_is_compatible	(GsPluginLoader *plugin_loader,
+							 GsApp *app);
+
+void		 gs_plugin_loader_run_adopt		(GsPluginLoader *plugin_loader,
+							 GsAppList *list);
+void		 gs_plugin_loader_emit_updates_changed	(GsPluginLoader *self);
+
 G_END_DECLS
