@@ -125,7 +125,13 @@ gboolean	gs_flatpak_search		(GsFlatpak		*self,
 						 gboolean		 interactive,
 						 GCancellable		*cancellable,
 						 GError			**error);
-gboolean	gs_flatpak_add_categories	(GsFlatpak		*self,
+gboolean	gs_flatpak_search_developer_apps(GsFlatpak		*self,
+						 const gchar * const	*values,
+						 GsAppList		*list,
+						 gboolean		 interactive,
+						 GCancellable		*cancellable,
+						 GError			**error);
+gboolean	gs_flatpak_refine_category_sizes(GsFlatpak		*self,
 						 GPtrArray		*list,
 						 gboolean		 interactive,
 						 GCancellable		*cancellable,
@@ -146,6 +152,13 @@ gboolean	gs_flatpak_add_featured		(GsFlatpak		*self,
 						 gboolean		 interactive,
 						 GCancellable		*cancellable,
 						 GError			**error);
+gboolean	gs_flatpak_add_deployment_featured
+						(GsFlatpak		*self,
+						 GsAppList		*list,
+						 gboolean		 interactive,
+						 const gchar *const	*deployments,
+						 GCancellable		*cancellable,
+						 GError			**error);
 gboolean	gs_flatpak_add_alternates	(GsFlatpak		*self,
 						 GsApp			*app,
 						 GsAppList		*list,
