@@ -124,7 +124,6 @@ static gchar *
 gs_fwupd_release_get_name (FwupdRelease *release)
 {
 	const gchar *name = fwupd_release_get_name (release);
-#if FWUPD_CHECK_VERSION(1,2,7)
 	GPtrArray *cats = fwupd_release_get_categories (release);
 
 	for (guint i = 0; i < cats->len; i++) {
@@ -206,19 +205,59 @@ gs_fwupd_release_get_name (FwupdRelease *release)
 			/* TRANSLATORS: Keyboard refers to an input device for typing */
 			return g_strdup_printf (_("%s Keyboard Update"), name);
 		}
+		if (g_strcmp0 (cat, "X-StorageController") == 0) {
+			/* TRANSLATORS: Storage Controller is typically a RAID or SAS adapter */
+			return g_strdup_printf (_("%s Storage Controller Update"), name);
+		}
+		if (g_strcmp0 (cat, "X-NetworkInterface") == 0) {
+			/* TRANSLATORS: Network Interface refers to the physical
+			 * PCI card, not the logical wired connection */
+			return g_strdup_printf (_("%s Network Interface Update"), name);
+		}
+		if (g_strcmp0 (cat, "X-VideoDisplay") == 0) {
+			/* TRANSLATORS: Video Display refers to the laptop internal display or
+			 * external monitor */
+			return g_strdup_printf (_("%s Display Update"), name);
+		}
+		if (g_strcmp0 (cat, "X-BaseboardManagementController") == 0) {
+			/* TRANSLATORS: BMC refers to baseboard management controller which
+			 * is the device that updates all the other firmware on the system */
+			return g_strdup_printf (_("%s BMC Update"), name);
+		}
+		if (g_strcmp0 (cat, "X-UsbReceiver") == 0) {
+			/* TRANSLATORS: Receiver refers to a radio device, e.g. a tiny Bluetooth
+			 * device that stays in the USB port so the wireless peripheral works */
+			return g_strdup_printf (_("%s USB Receiver Update"), name);
+		}
 	}
-#endif
 
 	/* default fallback */
 	return g_strdup (name);
 }
 
+static AsUrgencyKind
+gs_fwupd_release_urgency_to_as_urgency_kind (FwupdReleaseUrgency urgency)
+{
+	switch (urgency) {
+	case FWUPD_RELEASE_URGENCY_LOW:
+		return AS_URGENCY_KIND_LOW;
+	case FWUPD_RELEASE_URGENCY_MEDIUM:
+		return AS_URGENCY_KIND_MEDIUM;
+	case FWUPD_RELEASE_URGENCY_HIGH:
+		return AS_URGENCY_KIND_HIGH;
+	case FWUPD_RELEASE_URGENCY_CRITICAL:
+		return AS_URGENCY_KIND_CRITICAL;
+	case FWUPD_RELEASE_URGENCY_UNKNOWN:
+	default:
+		return AS_URGENCY_KIND_UNKNOWN;
+	}
+}
+
 void
 gs_fwupd_app_set_from_release (GsApp *app, FwupdRelease *rel)
 {
-#if FWUPD_CHECK_VERSION(1,5,6)
 	GPtrArray *locations = fwupd_release_get_locations (rel);
-#endif
+
 	if (fwupd_release_get_name (rel) != NULL) {
 		g_autofree gchar *tmp = gs_fwupd_release_get_name (rel);
 		gs_app_set_name (app, GS_APP_QUALITY_NORMAL, tmp);
@@ -232,8 +271,8 @@ gs_fwupd_app_set_from_release (GsApp *app, FwupdRelease *rel)
 				fwupd_release_get_homepage (rel));
 	}
 	if (fwupd_release_get_size (rel) != 0) {
-		gs_app_set_size_installed (app, 0);
-		gs_app_set_size_download (app, fwupd_release_get_size (rel));
+		gs_app_set_size_installed (app, GS_SIZE_TYPE_VALID, 0);
+		gs_app_set_size_download (app, GS_SIZE_TYPE_VALID, fwupd_release_get_size (rel));
 	}
 	if (fwupd_release_get_version (rel) != NULL)
 		gs_app_set_update_version (app, fwupd_release_get_version (rel));
@@ -241,7 +280,6 @@ gs_fwupd_app_set_from_release (GsApp *app, FwupdRelease *rel)
 		gs_app_set_license (app, GS_APP_QUALITY_NORMAL,
 				    fwupd_release_get_license (rel));
 	}
-#if FWUPD_CHECK_VERSION(1,5,6)
 	if (locations->len > 0) {
 		const gchar *uri = g_ptr_array_index (locations, 0);
 		/* typically the first URI will be the main HTTP mirror, and we
@@ -249,20 +287,12 @@ gs_fwupd_app_set_from_release (GsApp *app, FwupdRelease *rel)
 		gs_app_set_origin_hostname (app, uri);
 		gs_fwupd_app_set_update_uri (app, uri);
 	}
-#else
-	if (fwupd_release_get_uri (rel) != NULL) {
-		gs_app_set_origin_hostname (app,
-					    fwupd_release_get_uri (rel));
-		gs_fwupd_app_set_update_uri (app, fwupd_release_get_uri (rel));
-	}
-#endif
 	if (fwupd_release_get_description (rel) != NULL) {
 		g_autofree gchar *tmp = NULL;
 		tmp = as_markup_convert_simple (fwupd_release_get_description (rel), NULL);
 		if (tmp != NULL)
-			gs_app_set_update_details (app, tmp);
+			gs_app_set_update_details_text (app, tmp);
 	}
-#if FWUPD_CHECK_VERSION(1,3,3)
 	if (fwupd_release_get_detach_image (rel) != NULL) {
 		g_autoptr(AsScreenshot) ss = as_screenshot_new ();
 		g_autoptr(AsImage) im = as_image_new ();
@@ -271,8 +301,9 @@ gs_fwupd_app_set_from_release (GsApp *app, FwupdRelease *rel)
 		as_screenshot_set_kind (ss, AS_SCREENSHOT_KIND_DEFAULT);
 		as_screenshot_add_image (ss, im);
 		if (fwupd_release_get_detach_caption (rel) != NULL)
-			as_screenshot_set_caption (ss, NULL, fwupd_release_get_detach_caption (rel));
+			as_screenshot_set_caption (ss, fwupd_release_get_detach_caption (rel), NULL);
 		gs_app_set_action_screenshot (app, ss);
 	}
-#endif
+
+	gs_app_set_update_urgency (app, gs_fwupd_release_urgency_to_as_urgency_kind (fwupd_release_get_urgency (rel)));
 }
