@@ -12,6 +12,8 @@
 #include <gnome-software.h>
 #include <packagekit-glib2/packagekit.h>
 
+#include "gs-packagekit-helper.h"
+
 G_BEGIN_DECLS
 
 #define GS_TYPE_PACKAGEKIT_TASK (gs_packagekit_task_get_type ())
@@ -28,5 +30,9 @@ void		 gs_packagekit_task_setup	(GsPackagekitTask	*task,
 						 GsPluginAction		 action,
 						 gboolean		 interactive);
 GsPluginAction	 gs_packagekit_task_get_action	(GsPackagekitTask	*task);
+void		 gs_packagekit_task_take_helper	(GsPackagekitTask	*task,
+						 GsPackagekitHelper	*helper);
+GsPackagekitHelper *
+		 gs_packagekit_task_get_helper	(GsPackagekitTask	*task);
 
 G_END_DECLS
