@@ -20,8 +20,8 @@
 
 #include "config.h"
 
+#include <adwaita.h>
 #include <glib/gi18n.h>
-#include <handy.h>
 
 #include "gs-app-details-page.h"
 #include "gs-app-row.h"
@@ -31,6 +31,7 @@
 typedef enum {
 	PROP_APP = 1,
 	PROP_SHOW_BACK_BUTTON,
+	PROP_TITLE,
 } GsAppDetailsPageProperty;
 
 enum {
@@ -38,7 +39,7 @@ enum {
 	SIGNAL_LAST
 };
 
-static GParamSpec *obj_props[PROP_SHOW_BACK_BUTTON + 1] = { NULL, };
+static GParamSpec *obj_props[PROP_TITLE + 1] = { NULL, };
 
 static guint signals[SIGNAL_LAST] = { 0 };
 
@@ -47,15 +48,12 @@ struct _GsAppDetailsPage
 	GtkBox		 parent_instance;
 
 	GtkWidget	*back_button;
-	GtkWidget	*box_header;
 	GtkWidget	*header_bar;
-	GtkWidget	*image_icon;
 	GtkWidget	*label_details;
-	GtkWidget	*label_name;
-	GtkWidget	*label_summary;
-	GtkWidget	*permissions_section_box;
-	GtkWidget	*permissions_section_content;
-	GtkWidget	*scrolledwindow_details;
+	GtkWidget	*permissions_section;
+	GtkWidget	*permissions_section_list;
+	GtkWidget	*status_page;
+	AdwWindowTitle	*window_title;
 
 	GsApp		*app;  /* (owned) (nullable) */
 };
@@ -63,154 +61,159 @@ struct _GsAppDetailsPage
 G_DEFINE_TYPE (GsAppDetailsPage, gs_app_details_page, GTK_TYPE_BOX)
 
 static const struct {
-        GsAppPermissions permission;
+        GsAppPermissionsFlags permission;
         const char *title;
         const char *subtitle;
 } permission_display_data[] = {
-  { GS_APP_PERMISSIONS_NETWORK, N_("Network"), N_("Can communicate over the network") },
-  { GS_APP_PERMISSIONS_SYSTEM_BUS, N_("System Services"), N_("Can access D-Bus services on the system bus") },
-  { GS_APP_PERMISSIONS_SESSION_BUS, N_("Session Services"), N_("Can access D-Bus services on the session bus") },
-  { GS_APP_PERMISSIONS_DEVICES, N_("Devices"), N_("Can access system device files") },
-  { GS_APP_PERMISSIONS_HOME_FULL, N_("Home folder"), N_("Can view, edit and create files") },
-  { GS_APP_PERMISSIONS_HOME_READ, N_("Home folder"), N_("Can view files") },
-  { GS_APP_PERMISSIONS_FILESYSTEM_FULL, N_("File system"), N_("Can view, edit and create files") },
-  { GS_APP_PERMISSIONS_FILESYSTEM_READ, N_("File system"), N_("Can view files") },
-  { GS_APP_PERMISSIONS_DOWNLOADS_FULL, N_("Downloads folder"), N_("Can view, edit and create files") },
-  { GS_APP_PERMISSIONS_DOWNLOADS_READ, N_("Downloads folder"), N_("Can view files") },
-  { GS_APP_PERMISSIONS_SETTINGS, N_("Settings"), N_("Can view and change any settings") },
-  { GS_APP_PERMISSIONS_X11, N_("Legacy display system"), N_("Uses an old, insecure display system") },
-  { GS_APP_PERMISSIONS_ESCAPE_SANDBOX, N_("Sandbox escape"), N_("Can escape the sandbox and circumvent any other restrictions") },
+  { GS_APP_PERMISSIONS_FLAGS_NETWORK, N_("Network"), N_("Can communicate over the network") },
+  { GS_APP_PERMISSIONS_FLAGS_SYSTEM_BUS, N_("System Services"), N_("Can access D-Bus services on the system bus") },
+  { GS_APP_PERMISSIONS_FLAGS_SESSION_BUS, N_("Session Services"), N_("Can access D-Bus services on the session bus") },
+  { GS_APP_PERMISSIONS_FLAGS_DEVICES, N_("Devices"), N_("Can access system device files") },
+  { GS_APP_PERMISSIONS_FLAGS_HOME_FULL, N_("Home folder"), N_("Can view, edit and create files") },
+  { GS_APP_PERMISSIONS_FLAGS_HOME_READ, N_("Home folder"), N_("Can view files") },
+  { GS_APP_PERMISSIONS_FLAGS_FILESYSTEM_FULL, N_("File system"), N_("Can view, edit and create files") },
+  { GS_APP_PERMISSIONS_FLAGS_FILESYSTEM_READ, N_("File system"), N_("Can view files") },
+  /* The GS_APP_PERMISSIONS_FLAGS_FILESYSTEM_OTHER is used only as a flag, with actual files being part of the read/full lists */
+  { GS_APP_PERMISSIONS_FLAGS_DOWNLOADS_FULL, N_("Downloads folder"), N_("Can view, edit and create files") },
+  { GS_APP_PERMISSIONS_FLAGS_DOWNLOADS_READ, N_("Downloads folder"), N_("Can view files") },
+  { GS_APP_PERMISSIONS_FLAGS_SETTINGS, N_("Settings"), N_("Can view and change any settings") },
+  { GS_APP_PERMISSIONS_FLAGS_X11, N_("Legacy display system"), N_("Uses an old, insecure display system") },
+  { GS_APP_PERMISSIONS_FLAGS_ESCAPE_SANDBOX, N_("Sandbox escape"), N_("Can escape the sandbox and circumvent any other restrictions") },
 };
 
 static void
-destroy_cb (GtkWidget *widget, gpointer data)
+add_permissions_row (GsAppDetailsPage *page,
+		     const gchar *title,
+		     const gchar *subtitle,
+		     gboolean is_warning_row)
 {
-	gtk_widget_destroy (widget);
+	GtkWidget *row, *image;
+
+	row = adw_action_row_new ();
+	if (is_warning_row)
+		gtk_style_context_add_class (gtk_widget_get_style_context (row), "permission-row-warning");
+
+	image = gtk_image_new_from_icon_name ("dialog-warning-symbolic");
+	if (!is_warning_row)
+		gtk_widget_set_opacity (image, 0);
+
+#if ADW_CHECK_VERSION(1,2,0)
+	adw_preferences_row_set_use_markup (ADW_PREFERENCES_ROW (row), FALSE);
+#endif
+	adw_action_row_add_prefix (ADW_ACTION_ROW (row), image);
+	adw_preferences_row_set_title (ADW_PREFERENCES_ROW (row), title);
+	adw_action_row_set_subtitle (ADW_ACTION_ROW (row), subtitle);
+
+	gtk_list_box_append (GTK_LIST_BOX (page->permissions_section_list), row);
 }
 
 static void
-populate_permissions_section (GsAppDetailsPage *page, GsAppPermissions permissions)
+populate_permissions_filesystem (GsAppDetailsPage *page,
+				 const GPtrArray *titles, /* (element-type utf-8) */
+				 const gchar *subtitle,
+				 gboolean is_warning_row)
 {
-	gtk_container_foreach (GTK_CONTAINER (page->permissions_section_content), destroy_cb, NULL);
+	if (titles == NULL)
+		return;
+
+	for (guint i = 0; i < titles->len; i++) {
+		const gchar *title = g_ptr_array_index (titles, i);
+		add_permissions_row (page, title, subtitle, is_warning_row);
+	}
+}
+
+static void
+populate_permissions_section (GsAppDetailsPage *page,
+			      GsAppPermissions *permissions)
+{
+	GsAppPermissionsFlags flags = gs_app_permissions_get_flags (permissions);
+
+	gs_widget_remove_all (page->permissions_section_list, (GsRemoveFunc) gtk_list_box_remove);
 
 	for (gsize i = 0; i < G_N_ELEMENTS (permission_display_data); i++) {
-		GtkWidget *row, *image, *box, *label;
-
-		if ((permissions & permission_display_data[i].permission) == 0)
+		if ((flags & permission_display_data[i].permission) == 0)
 			continue;
 
-		row = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 12);
-		gtk_widget_show (row);
-		if ((permission_display_data[i].permission & ~MEDIUM_PERMISSIONS) != 0) {
-			gtk_style_context_add_class (gtk_widget_get_style_context (row), "permission-row-warning");
-		}
-
-		image = gtk_image_new_from_icon_name ("dialog-warning-symbolic", GTK_ICON_SIZE_MENU);
-		if ((permission_display_data[i].permission & ~MEDIUM_PERMISSIONS) == 0)
-			gtk_widget_set_opacity (image, 0);
-
-		gtk_widget_show (image);
-		gtk_container_add (GTK_CONTAINER (row), image);
-
-		box = gtk_box_new (GTK_ORIENTATION_VERTICAL, 0);
-		gtk_widget_show (box);
-		gtk_container_add (GTK_CONTAINER (row), box);
-
-		label = gtk_label_new (_(permission_display_data[i].title));
-		gtk_label_set_xalign (GTK_LABEL (label), 0);
-		gtk_widget_show (label);
-		gtk_container_add (GTK_CONTAINER (box), label);
-
-		label = gtk_label_new (_(permission_display_data[i].subtitle));
-		gtk_label_set_xalign (GTK_LABEL (label), 0);
-		gtk_style_context_add_class (gtk_widget_get_style_context (label), "dim-label");
-		gtk_widget_show (label);
-		gtk_container_add (GTK_CONTAINER (box), label);
-
-		gtk_container_add (GTK_CONTAINER (page->permissions_section_content), row);
+		add_permissions_row (page,
+			_(permission_display_data[i].title),
+			_(permission_display_data[i].subtitle),
+			(permission_display_data[i].permission & ~MEDIUM_PERMISSIONS) != 0);
 	}
+
+	populate_permissions_filesystem (page,
+		gs_app_permissions_get_filesystem_read (permissions),
+		_("Can view files"),
+		(GS_APP_PERMISSIONS_FLAGS_FILESYSTEM_READ & ~MEDIUM_PERMISSIONS) != 0);
+
+	populate_permissions_filesystem (page,
+		gs_app_permissions_get_filesystem_full (permissions),
+		_("Can view, edit and create files"),
+		(GS_APP_PERMISSIONS_FLAGS_FILESYSTEM_FULL & ~MEDIUM_PERMISSIONS) != 0);
 }
 
 static void
 set_updates_description_ui (GsAppDetailsPage *page, GsApp *app)
 {
-	AsComponentKind kind;
 	g_autoptr(GIcon) icon = NULL;
 	guint icon_size;
 	const gchar *update_details;
+	GdkDisplay *display;
+	g_autoptr (GtkIconPaintable) paintable = NULL;
 
 	/* FIXME support app == NULL */
 
 	/* set window title */
-	kind = gs_app_get_kind (app);
-	if (kind == AS_COMPONENT_KIND_GENERIC &&
-	    gs_app_get_special_kind (app) == GS_APP_SPECIAL_KIND_OS_UPDATE) {
-		hdy_header_bar_set_title (HDY_HEADER_BAR (page->header_bar),
-					  gs_app_get_name (app));
-	} else if (gs_app_get_source_default (app) != NULL &&
-		   gs_app_get_update_version (app) != NULL) {
-		g_autofree gchar *tmp = NULL;
-		/* Translators: This is the source and upgrade version of an
-		 * application, shown to the user when they view more detailed
-		 * information about pending updates. The source is of the form
-		 * ‘deja-dup’ (a package name) or
-		 * ‘app/org.gnome.Builder/x86_64/main’ (a flatpak ID), and the
-		 * version is of the form ‘40.4-1.fc34’ (a version number). */
-		tmp = g_strdup_printf (_("%s %s"),
-				       gs_app_get_source_default (app),
-				       gs_app_get_update_version (app));
-		hdy_header_bar_set_title (HDY_HEADER_BAR (page->header_bar), tmp);
-	} else if (gs_app_get_source_default (app) != NULL) {
-		hdy_header_bar_set_title (HDY_HEADER_BAR (page->header_bar),
-					  gs_app_get_source_default (app));
-	} else {
-		hdy_header_bar_set_title (HDY_HEADER_BAR (page->header_bar),
-					  gs_app_get_update_version (app));
-	}
+	adw_window_title_set_title (page->window_title, _("Update Details"));
+	g_object_notify_by_pspec (G_OBJECT (page), obj_props[PROP_TITLE]);
 
 	/* set update header */
-	gtk_widget_set_visible (page->box_header, kind == AS_COMPONENT_KIND_DESKTOP_APP);
-	update_details = gs_app_get_update_details (app);
+	update_details = gs_app_get_update_details_markup (app);
 	if (update_details == NULL) {
 		/* TRANSLATORS: this is where the packager did not write
 		 * a description for the update */
 		update_details = _("No update description available.");
 	}
-	gtk_label_set_label (GTK_LABEL (page->label_details), update_details);
-	gtk_label_set_label (GTK_LABEL (page->label_name), gs_app_get_name (app));
-	gtk_label_set_label (GTK_LABEL (page->label_summary), gs_app_get_summary (app));
+	gtk_label_set_markup (GTK_LABEL (page->label_details), update_details);
+	adw_status_page_set_title (ADW_STATUS_PAGE (page->status_page), gs_app_get_name (app));
+	adw_status_page_set_description (ADW_STATUS_PAGE (page->status_page), gs_app_get_summary (app));
 
 	/* set the icon; fall back to 64px if 96px isn’t available, which sometimes
 	 * happens at 2× scale factor (hi-DPI) */
 	icon_size = 96;
 	icon = gs_app_get_icon_for_size (app,
 					 icon_size,
-					 gtk_widget_get_scale_factor (page->image_icon),
+					 gtk_widget_get_scale_factor (GTK_WIDGET (page)),
 					 NULL);
 	if (icon == NULL) {
 		icon_size = 64;
 		icon = gs_app_get_icon_for_size (app,
 						 icon_size,
-						 gtk_widget_get_scale_factor (page->image_icon),
+						 gtk_widget_get_scale_factor (GTK_WIDGET (page)),
 						 NULL);
 	}
 	if (icon == NULL) {
 		icon_size = 96;
 		icon = gs_app_get_icon_for_size (app,
 						 icon_size,
-						 gtk_widget_get_scale_factor (page->image_icon),
+						 gtk_widget_get_scale_factor (GTK_WIDGET (page)),
 						 "system-component-application");
 	}
 
-	gtk_image_set_pixel_size (GTK_IMAGE (page->image_icon), icon_size);
-	gtk_image_set_from_gicon (GTK_IMAGE (page->image_icon), icon,
-				  GTK_ICON_SIZE_INVALID);
+	display = gdk_display_get_default ();
+	paintable = gtk_icon_theme_lookup_by_gicon (gtk_icon_theme_get_for_display (display),
+						    icon,
+						    icon_size,
+						    gtk_widget_get_scale_factor (GTK_WIDGET (page)),
+						    gtk_widget_get_direction (GTK_WIDGET (page)),
+						    GTK_ICON_LOOKUP_FORCE_REGULAR);
+	adw_status_page_set_paintable (ADW_STATUS_PAGE (page->status_page), GDK_PAINTABLE (paintable));
 
 	if (gs_app_has_quirk (app, GS_APP_QUIRK_NEW_PERMISSIONS)) {
-		gtk_widget_show (page->permissions_section_box);
-		populate_permissions_section (page, gs_app_get_update_permissions (app));
+		g_autoptr(GsAppPermissions) permissions = gs_app_dup_update_permissions (app);
+		gtk_widget_show (page->permissions_section);
+		populate_permissions_section (page, permissions);
 	} else {
-		gtk_widget_hide (page->permissions_section_box);
+		gtk_widget_hide (page->permissions_section);
 	}
 }
 
@@ -303,24 +306,6 @@ back_clicked_cb (GtkWidget *widget, GsAppDetailsPage *page)
 	g_signal_emit (page, signals[SIGNAL_BACK_CLICKED], 0);
 }
 
-static void
-scrollbar_mapped_cb (GtkWidget *sb, GtkScrolledWindow *swin)
-{
-	GtkWidget *frame;
-
-	frame = gtk_bin_get_child (GTK_BIN (gtk_bin_get_child (GTK_BIN (swin))));
-
-	if (gtk_widget_get_mapped (GTK_WIDGET (sb))) {
-		gtk_scrolled_window_set_shadow_type (swin, GTK_SHADOW_IN);
-		if (GTK_IS_FRAME (frame))
-			gtk_frame_set_shadow_type (GTK_FRAME (frame), GTK_SHADOW_NONE);
-	} else {
-		if (GTK_IS_FRAME (frame))
-			gtk_frame_set_shadow_type (GTK_FRAME (frame), GTK_SHADOW_IN);
-		gtk_scrolled_window_set_shadow_type (swin, GTK_SHADOW_NONE);
-	}
-}
-
 static void
 gs_app_details_page_dispose (GObject *object)
 {
@@ -343,6 +328,9 @@ gs_app_details_page_get_property (GObject *object, guint prop_id, GValue *value,
 	case PROP_SHOW_BACK_BUTTON:
 		g_value_set_boolean (value, gs_app_details_page_get_show_back_button (page));
 		break;
+	case PROP_TITLE:
+		g_value_set_string (value, adw_window_title_get_title (page->window_title));
+		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
 		break;
@@ -370,14 +358,7 @@ gs_app_details_page_set_property (GObject *object, guint prop_id, const GValue *
 static void
 gs_app_details_page_init (GsAppDetailsPage *page)
 {
-	GtkWidget *scrollbar;
-
 	gtk_widget_init_template (GTK_WIDGET (page));
-
-	scrollbar = gtk_scrolled_window_get_vscrollbar (GTK_SCROLLED_WINDOW (page->scrolledwindow_details));
-	g_signal_connect (scrollbar, "map", G_CALLBACK (scrollbar_mapped_cb), page->scrolledwindow_details);
-	g_signal_connect (scrollbar, "unmap", G_CALLBACK (scrollbar_mapped_cb), page->scrolledwindow_details);
-
 }
 
 static void
@@ -414,6 +395,18 @@ gs_app_details_page_class_init (GsAppDetailsPageClass *klass)
 				     TRUE,
 				     G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
 
+	/**
+	 * GsAppDetailsPage:title
+	 *
+	 * Read-only window title.
+	 *
+	 * Since: 42
+	 */
+	obj_props[PROP_TITLE] =
+		g_param_spec_string ("title", NULL, NULL,
+				     NULL,
+				     G_PARAM_READABLE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
+
 	g_object_class_install_properties (object_class, G_N_ELEMENTS (obj_props), obj_props);
 
 	/**
@@ -434,15 +427,12 @@ gs_app_details_page_class_init (GsAppDetailsPageClass *klass)
 	gtk_widget_class_set_template_from_resource (widget_class, "/org/gnome/Software/gs-app-details-page.ui");
 
 	gtk_widget_class_bind_template_child (widget_class, GsAppDetailsPage, back_button);
-	gtk_widget_class_bind_template_child (widget_class, GsAppDetailsPage, box_header);
 	gtk_widget_class_bind_template_child (widget_class, GsAppDetailsPage, header_bar);
-	gtk_widget_class_bind_template_child (widget_class, GsAppDetailsPage, image_icon);
 	gtk_widget_class_bind_template_child (widget_class, GsAppDetailsPage, label_details);
-	gtk_widget_class_bind_template_child (widget_class, GsAppDetailsPage, label_name);
-	gtk_widget_class_bind_template_child (widget_class, GsAppDetailsPage, label_summary);
-	gtk_widget_class_bind_template_child (widget_class, GsAppDetailsPage, permissions_section_box);
-	gtk_widget_class_bind_template_child (widget_class, GsAppDetailsPage, permissions_section_content);
-	gtk_widget_class_bind_template_child (widget_class, GsAppDetailsPage, scrolledwindow_details);
+	gtk_widget_class_bind_template_child (widget_class, GsAppDetailsPage, permissions_section);
+	gtk_widget_class_bind_template_child (widget_class, GsAppDetailsPage, permissions_section_list);
+	gtk_widget_class_bind_template_child (widget_class, GsAppDetailsPage, status_page);
+	gtk_widget_class_bind_template_child (widget_class, GsAppDetailsPage, window_title);
 	gtk_widget_class_bind_template_callback (widget_class, back_clicked_cb);
 }
 
