@@ -12,6 +12,10 @@
 #include <glib/gi18n.h>
 #include <gio/gdesktopappinfo.h>
 
+#ifndef TESTDATADIR
+#include "gs-application.h"
+#endif
+
 #include "gs-common.h"
 
 #ifdef HAVE_GSETTINGS_DESKTOP_SCHEMAS
@@ -20,83 +24,17 @@
 
 #include <langinfo.h>
 
-#define SPINNER_DELAY 500
-
-static gboolean
-fade_in (gpointer data)
+void
+gs_widget_remove_all (GtkWidget    *container,
+                      GsRemoveFunc  remove_func)
 {
-	GtkWidget *spinner = data;
-	gdouble opacity;
-
-	opacity = gtk_widget_get_opacity (spinner);
-	opacity = opacity + 0.1;
-	gtk_widget_set_opacity (spinner, opacity);
-
-	if (opacity >= 1.0) {
-		g_object_steal_data (G_OBJECT (spinner), "fade-timeout");
-		return G_SOURCE_REMOVE;
+	GtkWidget *child;
+	while ((child = gtk_widget_get_first_child (container)) != NULL) {
+		if (remove_func)
+			remove_func (container, child);
+		else
+			gtk_widget_unparent (child);
 	}
-	return G_SOURCE_CONTINUE;
-}
-
-static void
-remove_source (gpointer data)
-{
-	g_source_remove (GPOINTER_TO_UINT (data));
-}
-
-static gboolean
-start_spinning (gpointer data)
-{
-	GtkWidget *spinner = data;
-	guint id;
-
-	gtk_widget_set_opacity (spinner, 0);
-	gtk_spinner_start (GTK_SPINNER (spinner));
-	id = g_timeout_add (100, fade_in, spinner);
-	g_object_set_data_full (G_OBJECT (spinner), "fade-timeout",
-				GUINT_TO_POINTER (id), remove_source);
-
-	/* don't try to remove this source in the future */
-	g_object_steal_data (G_OBJECT (spinner), "start-timeout");
-	return G_SOURCE_REMOVE;
-}
-
-void
-gs_stop_spinner (GtkSpinner *spinner)
-{
-	g_object_set_data (G_OBJECT (spinner), "start-timeout", NULL);
-	gtk_spinner_stop (spinner);
-}
-
-void
-gs_start_spinner (GtkSpinner *spinner)
-{
-	gboolean active;
-	guint id;
-
-	/* Don't do anything if it's already spinning */
-	g_object_get (spinner, "active", &active, NULL);
-	if (active || g_object_get_data (G_OBJECT (spinner), "start-timeout") != NULL)
-		return;
-
-	gtk_widget_set_opacity (GTK_WIDGET (spinner), 0);
-	id = g_timeout_add (SPINNER_DELAY, start_spinning, spinner);
-	g_object_set_data_full (G_OBJECT (spinner), "start-timeout",
-				GUINT_TO_POINTER (id), remove_source);
-}
-
-static void
-remove_all_cb (GtkWidget *widget, gpointer user_data)
-{
-	GtkContainer *container = GTK_CONTAINER (user_data);
-	gtk_container_remove (container, widget);
-}
-
-void
-gs_container_remove_all (GtkContainer *container)
-{
-	gtk_container_foreach (container, remove_all_cb, container);
 }
 
 static void
@@ -143,7 +81,7 @@ gs_app_notify_installed (GsApp *app)
 		    gs_app_get_special_kind (app) == GS_APP_SPECIAL_KIND_OS_UPDATE) {
 			/* TRANSLATORS: this is the summary of a notification that OS updates
 			* have been successfully installed */
-			summary = g_strdup (_("OS updates are now installed"));
+			summary = g_strdup (_("System updates are now installed"));
 			/* TRANSLATORS: this is the body of a notification that OS updates
 			* have been successfully installed */
 			body = _("Recently installed updates are available to review");
@@ -169,14 +107,20 @@ gs_app_notify_installed (GsApp *app)
 						       "app.reboot", NULL);
 	} else if (gs_app_get_kind (app) == AS_COMPONENT_KIND_DESKTOP_APP) {
 		/* TRANSLATORS: this is button that opens the newly installed application */
+		g_autoptr(GsPlugin) plugin = gs_app_dup_management_plugin (app);
+		const gchar *plugin_name = (plugin != NULL) ? gs_plugin_get_name (plugin) : "";
 		g_notification_add_button_with_target (n, _("Launch"),
 						       "app.launch", "(ss)",
 						       gs_app_get_id (app),
-						       gs_app_get_management_plugin (app));
+						       plugin_name);
 	}
 	g_notification_set_default_action_and_target  (n, "app.details", "(ss)",
 						       gs_app_get_unique_id (app), "");
+	#ifdef TESTDATADIR
 	g_application_send_notification (g_application_get_default (), "installed", n);
+	#else
+	gs_application_send_notification (GS_APPLICATION (g_application_get_default ()), "installed", n, 24 * 60);
+	#endif
 }
 
 typedef enum {
@@ -185,14 +129,52 @@ typedef enum {
 	GS_APP_LICENSE_PATENT_CONCERN	= 2
 } GsAppLicenseHint;
 
+typedef struct
+{
+	gint response_id;
+	GMainLoop *loop;
+} RunInfo;
+
+static void
+shutdown_loop (RunInfo *run_info)
+{
+	if (g_main_loop_is_running (run_info->loop))
+		g_main_loop_quit (run_info->loop);
+}
+
+static void
+unmap_cb (GtkDialog *dialog,
+          RunInfo   *run_info)
+{
+	shutdown_loop (run_info);
+}
+
+static void
+response_cb (GtkDialog *dialog,
+             gint       response_id,
+             RunInfo   *run_info)
+{
+	run_info->response_id = response_id;
+	gtk_window_destroy (GTK_WINDOW (dialog));
+	shutdown_loop (run_info);
+}
+
+static gboolean
+close_requested_cb (GtkDialog *dialog,
+                    RunInfo   *run_info)
+{
+	shutdown_loop (run_info);
+	return GDK_EVENT_PROPAGATE;
+}
+
 GtkResponseType
 gs_app_notify_unavailable (GsApp *app, GtkWindow *parent)
 {
 	GsAppLicenseHint hint = GS_APP_LICENSE_FREE;
-	GtkResponseType response;
 	GtkWidget *dialog;
 	const gchar *license;
 	gboolean already_enabled = FALSE;	/* FIXME */
+	g_autofree gchar *origin_ui = NULL;
 	guint i;
 	struct {
 		const gchar	*str;
@@ -207,6 +189,11 @@ gs_app_notify_unavailable (GsApp *app, GtkWindow *parent)
 	g_autoptr(GString) body = NULL;
 	g_autoptr(GString) title = NULL;
 
+	RunInfo run_info = {
+		GTK_RESPONSE_NONE,
+		NULL,
+	};
+
 	/* this is very crude */
 	license = gs_app_get_license (app);
 	if (license != NULL) {
@@ -242,6 +229,8 @@ gs_app_notify_unavailable (GsApp *app, GtkWindow *parent)
 	gtk_message_dialog_set_markup (GTK_MESSAGE_DIALOG (dialog), title->str);
 
 	body = g_string_new ("");
+	origin_ui = gs_app_dup_origin_ui (app, TRUE);
+
 	if (hint & GS_APP_LICENSE_NONFREE) {
 		g_string_append_printf (body,
 					/* TRANSLATORS: the replacements are as follows:
@@ -252,7 +241,7 @@ gs_app_notify_unavailable (GsApp *app, GtkWindow *parent)
 					  "free and open source software</a>, "
 					  "and is provided by “%s”."),
 					gs_app_get_name (app),
-					gs_app_get_origin (app));
+					origin_ui);
 	} else {
 		g_string_append_printf (body,
 					/* TRANSLATORS: the replacements are as follows:
@@ -260,7 +249,7 @@ gs_app_notify_unavailable (GsApp *app, GtkWindow *parent)
 					 * 2. Software repository name, e.g. fedora-optional */
 					_("%s is provided by “%s”."),
 					gs_app_get_name (app),
-					gs_app_get_origin (app));
+					origin_ui);
 	}
 
 	/* tell the use what needs to be done */
@@ -302,13 +291,25 @@ gs_app_notify_unavailable (GsApp *app, GtkWindow *parent)
 				       _("Enable and Install"),
 				       GTK_RESPONSE_OK);
 	}
-	response = gtk_dialog_run (GTK_DIALOG (dialog));
-	if (response == GTK_RESPONSE_YES) {
-		response = GTK_RESPONSE_OK;
+
+
+	/* Run */
+	if (!gtk_widget_get_visible (dialog))
+		gtk_window_present (GTK_WINDOW (dialog));
+
+	g_signal_connect (dialog, "close-request", G_CALLBACK (close_requested_cb), &run_info);
+	g_signal_connect (dialog, "response", G_CALLBACK (response_cb), &run_info);
+	g_signal_connect (dialog, "unmap", G_CALLBACK (unmap_cb), &run_info);
+
+	run_info.loop = g_main_loop_new (NULL, FALSE);
+	g_main_loop_run (run_info.loop);
+	g_clear_pointer (&run_info.loop, g_main_loop_unref);
+
+	if (run_info.response_id == GTK_RESPONSE_YES) {
+		run_info.response_id = GTK_RESPONSE_OK;
 		g_settings_set_boolean (settings, "prompt-for-nonfree", FALSE);
 	}
-	gtk_widget_destroy (dialog);
-	return response;
+	return run_info.response_id;
 }
 
 gboolean
@@ -329,9 +330,12 @@ gs_utils_widget_css_parsing_error_cb (GtkCssProvider *provider,
 				      GError *error,
 				      gpointer user_data)
 {
-	g_warning ("CSS parse error %u:%u: %s",
-		   gtk_css_section_get_start_line (section),
-		   gtk_css_section_get_start_position (section),
+	const GtkCssLocation *start_location;
+
+	start_location = gtk_css_section_get_start_location (section);
+	g_warning ("CSS parse error %" G_GSIZE_FORMAT ":%" G_GSIZE_FORMAT ": %s",
+		   start_location->lines + 1,
+		   start_location->line_chars,
 		   error->message);
 }
 
@@ -445,24 +449,16 @@ gs_utils_widget_set_css (GtkWidget *widget, GtkCssProvider **provider, const gch
 	gtk_style_context_add_class (context, class_name);
 
 	/* set up custom provider and store on the widget */
-	gtk_css_provider_load_from_data (*provider, str->str, -1, NULL);
+	gtk_css_provider_load_from_data (*provider, str->str, -1);
 	gtk_style_context_add_provider (context, GTK_STYLE_PROVIDER (*provider),
 					GTK_STYLE_PROVIDER_PRIORITY_APPLICATION);
 }
 
 static void
-do_not_expand (GtkWidget *child, gpointer data)
-{
-	gtk_container_child_set (GTK_CONTAINER (gtk_widget_get_parent (child)),
-				 child, "expand", FALSE, "fill", FALSE, NULL);
-}
-
-static gboolean
-unset_focus (GtkWidget *widget, GdkEvent *event, gpointer data)
+unset_focus (GtkWidget *widget, gpointer data)
 {
 	if (GTK_IS_WINDOW (widget))
 		gtk_window_set_focus (GTK_WINDOW (widget), NULL);
-	return FALSE;
 }
 
 /**
@@ -473,12 +469,14 @@ unset_focus (GtkWidget *widget, GdkEvent *event, gpointer data)
  * Inserts a widget displaying the detailed message into the message dialog.
  */
 static void
-insert_details_widget (GtkMessageDialog *dialog, const gchar *details)
+insert_details_widget (GtkMessageDialog *dialog,
+		       const gchar *details,
+		       gboolean add_prefix)
 {
 	GtkWidget *message_area, *sw, *label;
-	GtkWidget *box, *tv;
+	GtkWidget *tv;
+	GtkWidget *child;
 	GtkTextBuffer *buffer;
-	GList *children;
 	g_autoptr(GString) msg = NULL;
 
 	g_assert (GTK_IS_MESSAGE_DIALOG (dialog));
@@ -486,38 +484,34 @@ insert_details_widget (GtkMessageDialog *dialog, const gchar *details)
 
 	gtk_window_set_resizable (GTK_WINDOW (dialog), TRUE);
 
-	msg = g_string_new ("");
-	g_string_append_printf (msg, "%s\n\n%s",
-	                        /* TRANSLATORS: these are show_detailed_error messages from the
-	                         * package manager no mortal is supposed to understand,
-	                         * but google might know what they mean */
-	                        _("Detailed errors from the package manager follow:"),
-	                        details);
+	if (add_prefix) {
+		msg = g_string_new ("");
+		g_string_append_printf (msg, "%s\n\n%s",
+					/* TRANSLATORS: these are show_detailed_error messages from the
+					 * package manager no mortal is supposed to understand,
+					 * but google might know what they mean */
+					_("Detailed errors from the package manager follow:"),
+					details);
+	}
 
 	message_area = gtk_message_dialog_get_message_area (dialog);
 	g_assert (GTK_IS_BOX (message_area));
-	/* make the hbox expand */
-	box = gtk_widget_get_parent (message_area);
-	gtk_container_child_set (GTK_CONTAINER (gtk_widget_get_parent (box)), box,
-	                         "expand", TRUE, "fill", TRUE, NULL);
-	/* make the labels not expand */
-	gtk_container_foreach (GTK_CONTAINER (message_area), do_not_expand, NULL);
 
 	/* Find the secondary label and set its width_chars.   */
 	/* Otherwise the label will tend to expand vertically. */
-	children = gtk_container_get_children (GTK_CONTAINER (message_area));
-	if (children && children->next && GTK_IS_LABEL (children->next->data)) {
-		gtk_label_set_width_chars (GTK_LABEL (children->next->data), 40);
+	child = gtk_widget_get_first_child (message_area);
+	if (child) {
+		GtkWidget *next = gtk_widget_get_next_sibling (child);
+		if (next && GTK_IS_LABEL (next))
+			gtk_label_set_width_chars (GTK_LABEL (next), 40);
 	}
 
 	label = gtk_label_new (_("Details"));
 	gtk_widget_set_halign (label, GTK_ALIGN_START);
 	gtk_widget_set_visible (label, TRUE);
-	gtk_container_add (GTK_CONTAINER (message_area), label);
+	gtk_box_append (GTK_BOX (message_area), label);
 
-	sw = gtk_scrolled_window_new (NULL, NULL);
-	gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (sw),
-	                                     GTK_SHADOW_IN);
+	sw = gtk_scrolled_window_new ();
 	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (sw),
 	                                GTK_POLICY_NEVER,
 	                                GTK_POLICY_AUTOMATIC);
@@ -530,15 +524,14 @@ insert_details_widget (GtkMessageDialog *dialog, const gchar *details)
 	gtk_text_view_set_wrap_mode (GTK_TEXT_VIEW (tv), GTK_WRAP_WORD);
 	gtk_style_context_add_class (gtk_widget_get_style_context (tv),
 	                             "update-failed-details");
-	gtk_text_buffer_set_text (buffer, msg->str, -1);
+	gtk_text_buffer_set_text (buffer, msg ? msg->str : details, -1);
 	gtk_widget_set_visible (tv, TRUE);
 
-	gtk_container_add (GTK_CONTAINER (sw), tv);
+	gtk_scrolled_window_set_child (GTK_SCROLLED_WINDOW (sw), tv);
 	gtk_widget_set_vexpand (sw, TRUE);
-	gtk_container_add (GTK_CONTAINER (message_area), sw);
-	gtk_container_child_set (GTK_CONTAINER (message_area), sw, "pack-type", GTK_PACK_END, NULL);
+	gtk_box_append (GTK_BOX (message_area), sw);
 
-	g_signal_connect (dialog, "map-event", G_CALLBACK (unset_focus), NULL);
+	g_signal_connect (dialog, "map", G_CALLBACK (unset_focus), NULL);
 }
 
 /**
@@ -566,14 +559,76 @@ gs_utils_show_error_dialog (GtkWindow *parent,
 	gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (dialog),
 	                                          "%s", msg);
 	if (details != NULL)
-		insert_details_widget (GTK_MESSAGE_DIALOG (dialog), details);
+		insert_details_widget (GTK_MESSAGE_DIALOG (dialog), details, TRUE);
 
 	g_signal_connect_swapped (dialog, "response",
-	                          G_CALLBACK (gtk_widget_destroy),
+	                          G_CALLBACK (gtk_window_destroy),
 	                          dialog);
 	gtk_widget_show (dialog);
 }
 
+/**
+ * gs_utils_ask_user_accepts:
+ * @parent: (nullable): modal parent, or %NULL for none
+ * @title: the title for the dialog
+ * @msg: the message for the dialog
+ * @details: (nullable): the detailed error message, or %NULL for none
+ * @accept_label: (nullable): a label of the 'accept' button, or %NULL to use 'Accept'
+ *
+ * Shows a modal question dialog for displaying an accept/cancel question to the user.
+ *
+ * Returns: whether the user accepted the question
+ *
+ * Since: 42
+ **/
+gboolean
+gs_utils_ask_user_accepts (GtkWindow *parent,
+			   const gchar *title,
+			   const gchar *msg,
+			   const gchar *details,
+			   const gchar *accept_label)
+{
+	GtkWidget *dialog;
+	RunInfo run_info;
+
+	g_return_val_if_fail (parent == NULL || GTK_IS_WINDOW (parent), FALSE);
+	g_return_val_if_fail (title != NULL, FALSE);
+	g_return_val_if_fail (msg != NULL, FALSE);
+
+	if (accept_label == NULL || *accept_label == '\0') {
+		/* Translators: an accept button label, in a Cancel/Accept dialog */
+		accept_label = _("_Accept");
+	}
+
+	dialog = gtk_message_dialog_new_with_markup (parent,
+	                                             GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
+	                                             GTK_MESSAGE_QUESTION,
+	                                             GTK_BUTTONS_NONE,
+	                                             "<big><b>%s</b></big>", title);
+	gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (dialog),
+	                                          "%s", msg);
+	if (details != NULL)
+		insert_details_widget (GTK_MESSAGE_DIALOG (dialog), details, FALSE);
+	gtk_dialog_add_button (GTK_DIALOG (dialog), _("_Cancel"), GTK_RESPONSE_CANCEL);
+	gtk_dialog_add_button (GTK_DIALOG (dialog), accept_label, GTK_RESPONSE_OK);
+
+	run_info.response_id = GTK_RESPONSE_NONE;
+	run_info.loop = g_main_loop_new (NULL, FALSE);
+
+	/* Run */
+	if (!gtk_widget_get_visible (dialog))
+		gtk_window_present (GTK_WINDOW (dialog));
+
+	g_signal_connect (dialog, "close-request", G_CALLBACK (close_requested_cb), &run_info);
+	g_signal_connect (dialog, "response", G_CALLBACK (response_cb), &run_info);
+	g_signal_connect (dialog, "unmap", G_CALLBACK (unmap_cb), &run_info);
+
+	g_main_loop_run (run_info.loop);
+	g_clear_pointer (&run_info.loop, g_main_loop_unref);
+
+	return run_info.response_id == GTK_RESPONSE_OK;
+}
+
 /**
  * gs_utils_get_error_value:
  * @error: A GError
@@ -720,7 +775,11 @@ gs_utils_reboot_notify (GsAppList *list,
 	g_notification_add_button_with_target (n, _("Restart"), "app.reboot", NULL);
 	g_notification_set_default_action_and_target (n, "app.set-mode", "s", "updates");
 	g_notification_set_priority (n, G_NOTIFICATION_PRIORITY_URGENT);
+	#ifdef TESTDATADIR
 	g_application_send_notification (g_application_get_default (), "restart-required", n);
+	#else
+	gs_application_send_notification (GS_APPLICATION (g_application_get_default ()), "restart-required", n, 0);
+	#endif
 }
 
 /**
@@ -839,16 +898,270 @@ gs_utils_reboot_call_done_cb (GObject *source,
 			      GAsyncResult *res,
 			      gpointer user_data)
 {
-	g_autoptr(GError) error = NULL;
-	g_autoptr(GVariant) retval = NULL;
+	g_autoptr(GError) local_error = NULL;
 
 	/* get result */
-	retval = g_dbus_connection_call_finish (G_DBUS_CONNECTION (source), res, &error);
-	if (retval != NULL)
+	if (gs_utils_invoke_reboot_finish (source, res, &local_error))
 		return;
-	if (error != NULL) {
-		g_warning ("Calling org.gnome.SessionManager.Reboot failed: %s",
-			   error->message);
+	if (g_error_matches (local_error, G_IO_ERROR, G_IO_ERROR_CANCELLED))
+		g_debug ("Calling reboot had been cancelled");
+	else if (local_error != NULL)
+		g_warning ("Calling reboot failed: %s", local_error->message);
+}
+
+static void
+gs_utils_invoke_reboot_ready3_cb (GObject *source_object,
+				  GAsyncResult *result,
+				  gpointer user_data)
+{
+	g_autoptr(GTask) task = user_data;
+	g_autoptr(GVariant) ret_val = NULL;
+	g_autoptr(GError) local_error = NULL;
+
+	ret_val = g_dbus_connection_call_finish (G_DBUS_CONNECTION (source_object), result, &local_error);
+	if (ret_val != NULL) {
+		g_task_return_boolean (task, TRUE);
+	} else {
+		const gchar *method_name = g_task_get_task_data (task);
+		g_dbus_error_strip_remote_error (local_error);
+		g_prefix_error (&local_error, "Failed to call %s: ", method_name);
+		g_task_return_error (task, g_steal_pointer (&local_error));
+	}
+}
+
+static void
+gs_utils_invoke_reboot_ready2_got_session_bus_cb (GObject *source_object,
+						  GAsyncResult *result,
+						  gpointer user_data)
+{
+	g_autoptr(GTask) task = user_data;
+	g_autoptr(GDBusConnection) bus = NULL;
+	g_autoptr(GError) local_error = NULL;
+	GCancellable *cancellable;
+
+	bus = g_bus_get_finish (result, &local_error);
+	if (bus == NULL) {
+		g_dbus_error_strip_remote_error (local_error);
+		g_prefix_error_literal (&local_error, "Failed to get D-Bus session bus: ");
+		g_task_return_error (task, g_steal_pointer (&local_error));
+		return;
+	}
+
+	cancellable = g_task_get_cancellable (task);
+
+	/* Make sure file buffers are written to the disk before invoking reboot */
+	sync ();
+
+	g_task_set_task_data (task, (gpointer) "org.gnome.SessionManager.Reboot", NULL);
+	g_dbus_connection_call (bus,
+				"org.gnome.SessionManager",
+				"/org/gnome/SessionManager",
+				"org.gnome.SessionManager",
+				"Reboot",
+				NULL, NULL, G_DBUS_CALL_FLAGS_NONE,
+				G_MAXINT, cancellable,
+				gs_utils_invoke_reboot_ready3_cb,
+				g_steal_pointer (&task));
+}
+
+static void
+gs_utils_invoke_reboot_ready2_cb (GObject *source_object,
+				  GAsyncResult *result,
+				  gpointer user_data)
+{
+	g_autoptr(GTask) task = user_data;
+	g_autoptr(GVariant) ret_val = NULL;
+	g_autoptr(GError) local_error = NULL;
+
+	ret_val = g_dbus_connection_call_finish (G_DBUS_CONNECTION (source_object), result, &local_error);
+	if (ret_val != NULL) {
+		g_task_return_boolean (task, TRUE);
+	} else {
+		g_autoptr(GDBusConnection) bus = NULL;
+		GCancellable *cancellable;
+		const gchar *method_name = g_task_get_task_data (task);
+
+		g_dbus_error_strip_remote_error (local_error);
+		g_prefix_error (&local_error, "Failed to call %s: ", method_name);
+
+		if (g_error_matches (local_error, G_IO_ERROR, G_IO_ERROR_CANCELLED)) {
+			g_task_return_error (task, g_steal_pointer (&local_error));
+			return;
+		}
+
+		g_debug ("%s", local_error->message);
+		g_clear_error (&local_error);
+
+		cancellable = g_task_get_cancellable (task);
+
+		g_bus_get (G_BUS_TYPE_SESSION, cancellable,
+			   gs_utils_invoke_reboot_ready2_got_session_bus_cb,
+			   g_steal_pointer (&task));
+	}
+}
+
+static void
+gs_utils_invoke_reboot_ready1_got_system_bus_cb (GObject *source_object,
+						  GAsyncResult *result,
+						  gpointer user_data)
+{
+	g_autoptr(GTask) task = user_data;
+	g_autoptr(GDBusConnection) bus = NULL;
+	g_autoptr(GError) local_error = NULL;
+	GCancellable *cancellable;
+
+	bus = g_bus_get_finish (result, &local_error);
+	if (bus == NULL) {
+		g_dbus_error_strip_remote_error (local_error);
+		g_prefix_error_literal (&local_error, "Failed to get D-Bus system bus: ");
+		g_task_return_error (task, g_steal_pointer (&local_error));
+		return;
+	}
+
+	cancellable = g_task_get_cancellable (task);
+
+	/* Make sure file buffers are written to the disk before invoking reboot */
+	sync ();
+
+	g_task_set_task_data (task, (gpointer) "org.freedesktop.login1.Manager.Reboot", NULL);
+	g_dbus_connection_call (bus,
+				"org.freedesktop.login1",
+				"/org/freedesktop/login1",
+				"org.freedesktop.login1.Manager",
+				"Reboot",
+				g_variant_new ("(b)", TRUE), /* interactive */
+				NULL, G_DBUS_CALL_FLAGS_NONE,
+				G_MAXINT, cancellable,
+				gs_utils_invoke_reboot_ready2_cb,
+				g_steal_pointer (&task));
+}
+
+static void
+gs_utils_invoke_reboot_ready1_cb (GObject *source_object,
+				  GAsyncResult *result,
+				  gpointer user_data)
+{
+	g_autoptr(GTask) task = user_data;
+	g_autoptr(GVariant) ret_val = NULL;
+	g_autoptr(GError) local_error = NULL;
+
+	ret_val = g_dbus_connection_call_finish (G_DBUS_CONNECTION (source_object), result, &local_error);
+	if (ret_val != NULL) {
+		g_task_return_boolean (task, TRUE);
+	} else {
+		GCancellable *cancellable;
+		const gchar *method_name = g_task_get_task_data (task);
+
+		g_dbus_error_strip_remote_error (local_error);
+		g_prefix_error (&local_error, "Failed to call %s: ", method_name);
+
+		if (g_error_matches (local_error, G_IO_ERROR, G_IO_ERROR_CANCELLED)) {
+			g_task_return_error (task, g_steal_pointer (&local_error));
+			return;
+		}
+
+		g_debug ("%s", local_error->message);
+		g_clear_error (&local_error);
+
+		cancellable = g_task_get_cancellable (task);
+
+		g_bus_get (G_BUS_TYPE_SYSTEM, cancellable,
+			   gs_utils_invoke_reboot_ready1_got_system_bus_cb,
+			   g_steal_pointer (&task));
+	}
+}
+
+static void
+gs_utils_invoke_reboot_got_session_bus_cb (GObject *source_object,
+					   GAsyncResult *result,
+					   gpointer user_data)
+{
+	g_autoptr(GTask) task = user_data;
+	g_autoptr(GDBusConnection) bus = NULL;
+	g_autoptr(GError) local_error = NULL;
+	GCancellable *cancellable;
+	const gchar *xdg_desktop;
+	gboolean call_session_manager = FALSE;
+
+	bus = g_bus_get_finish (result, &local_error);
+	if (bus == NULL) {
+		g_dbus_error_strip_remote_error (local_error);
+		g_prefix_error_literal (&local_error, "Failed to get D-Bus session bus: ");
+		g_task_return_error (task, g_steal_pointer (&local_error));
+		return;
+	}
+
+	/* Make sure file buffers are written to the disk before invoking reboot */
+	sync ();
+
+	cancellable = g_task_get_cancellable (task);
+
+	xdg_desktop = g_getenv ("XDG_CURRENT_DESKTOP");
+	if (xdg_desktop != NULL) {
+		if (strstr (xdg_desktop, "KDE")) {
+			g_task_set_task_data (task, (gpointer) "org.kde.Shutdown.logoutAndReboot", NULL);
+			g_dbus_connection_call (bus,
+						"org.kde.Shutdown",
+						"/Shutdown",
+						"org.kde.Shutdown",
+						"logoutAndReboot",
+						NULL, NULL, G_DBUS_CALL_FLAGS_NONE,
+						G_MAXINT, cancellable,
+						gs_utils_invoke_reboot_ready1_cb,
+						g_steal_pointer (&task));
+		} else if (strstr (xdg_desktop, "LXDE")) {
+			g_task_set_task_data (task, (gpointer) "org.lxde.SessionManager.RequestReboot", NULL);
+			g_dbus_connection_call (bus,
+						"org.lxde.SessionManager",
+						"/org/lxde/SessionManager",
+						"org.lxde.SessionManager",
+						"RequestReboot",
+						NULL, NULL, G_DBUS_CALL_FLAGS_NONE,
+						G_MAXINT, cancellable,
+						gs_utils_invoke_reboot_ready1_cb,
+						g_steal_pointer (&task));
+		} else if (strstr (xdg_desktop, "MATE")) {
+			g_task_set_task_data (task, (gpointer) "org.gnome.SessionManager.RequestReboot", NULL);
+			g_dbus_connection_call (bus,
+						"org.gnome.SessionManager",
+						"/org/gnome/SessionManager",
+						"org.gnome.SessionManager",
+						"RequestReboot",
+						NULL, NULL, G_DBUS_CALL_FLAGS_NONE,
+						G_MAXINT, cancellable,
+						gs_utils_invoke_reboot_ready1_cb,
+						g_steal_pointer (&task));
+		} else if (strstr (xdg_desktop, "XFCE")) {
+			g_task_set_task_data (task, (gpointer) "org.xfce.Session.Manager.Restart", NULL);
+			g_dbus_connection_call (bus,
+						"org.xfce.SessionManager",
+						"/org/xfce/SessionManager",
+						"org.xfce.Session.Manager",
+						"Restart",
+						g_variant_new ("(b)", TRUE), /* allow_save */
+						NULL, G_DBUS_CALL_FLAGS_NONE,
+						G_MAXINT, cancellable,
+						gs_utils_invoke_reboot_ready1_cb,
+						g_steal_pointer (&task));
+		} else {
+			/* Let the "GNOME" and "X-Cinnamon" be the default */
+			call_session_manager = TRUE;
+		}
+	} else {
+		call_session_manager = TRUE;
+	}
+
+	if (call_session_manager) {
+		g_task_set_task_data (task, (gpointer) "org.gnome.SessionManager.Reboot", NULL);
+		g_dbus_connection_call (bus,
+					"org.gnome.SessionManager",
+					"/org/gnome/SessionManager",
+					"org.gnome.SessionManager",
+					"Reboot",
+					NULL, NULL, G_DBUS_CALL_FLAGS_NONE,
+					G_MAXINT, cancellable,
+					gs_utils_invoke_reboot_ready3_cb,
+					g_steal_pointer (&task));
 	}
 }
 
@@ -858,9 +1171,8 @@ gs_utils_reboot_call_done_cb (GObject *source,
  * @ready_callback: (nullable): a callback to be called after the call is finished, or %NULL
  * @user_data: user data for the @ready_callback
  *
- * Asynchronously invokes a reboot request using D-Bus. The @ready_callback should
- * use g_dbus_connection_call_finish (G_DBUS_CONNECTION (source), result, &error);
- * to get the result of the operation.
+ * Asynchronously invokes a reboot request. Finish the operation
+ * with gs_utils_invoke_reboot_finish().
  *
  * When the @ready_callback is %NULL, a default callback is used, which shows
  * a runtime warning (using g_warning) on the console when the call fails.
@@ -872,19 +1184,78 @@ gs_utils_invoke_reboot_async (GCancellable *cancellable,
 			      GAsyncReadyCallback ready_callback,
 			      gpointer user_data)
 {
-	g_autoptr(GDBusConnection) bus = NULL;
-	bus = g_bus_get_sync (G_BUS_TYPE_SESSION, NULL, NULL);
+	g_autoptr(GTask) task = NULL;
 
 	if (!ready_callback)
 		ready_callback = gs_utils_reboot_call_done_cb;
 
-	g_dbus_connection_call (bus,
-				"org.gnome.SessionManager",
-				"/org/gnome/SessionManager",
-				"org.gnome.SessionManager",
-				"Reboot",
-				NULL, NULL, G_DBUS_CALL_FLAGS_NONE,
-				G_MAXINT, cancellable,
-				ready_callback,
-				user_data);
+	task = g_task_new (NULL, cancellable, ready_callback, user_data);
+	g_task_set_source_tag (task, gs_utils_invoke_reboot_async);
+
+	g_bus_get (G_BUS_TYPE_SESSION, cancellable,
+		   gs_utils_invoke_reboot_got_session_bus_cb,
+		   g_steal_pointer (&task));
+}
+
+/**
+ * gs_utils_invoke_reboot_finish:
+ * @source_object: the source object provided in the ready callback
+ * @result: the result object provided in the ready callback
+ * @error: a #GError, or %NULL
+ *
+ * Finishes gs_utils_invoke_reboot_async() call.
+ *
+ * Returns: Whether succeeded. If failed, the @error is set.
+ *
+ * Since: 43
+ **/
+gboolean
+gs_utils_invoke_reboot_finish (GObject *source_object,
+			       GAsyncResult *result,
+			       GError **error)
+{
+	g_return_val_if_fail (G_IS_TASK (result), FALSE);
+	g_return_val_if_fail (g_task_is_valid (result, source_object), FALSE);
+	g_return_val_if_fail (g_task_get_source_tag (G_TASK (result)) == gs_utils_invoke_reboot_async, FALSE);
+
+	return g_task_propagate_boolean (G_TASK (result), error);
+}
+
+/**
+ * gs_utils_format_size:
+ * @size_bytes: size to format, in bytes
+ * @out_is_markup: (out) (not nullable): stores whther the returned string is a markup
+ *
+ * Similar to `g_format_size`, only splits the value and the unit into
+ * separate parts and draws the unit with a smaller font, in case
+ * the relevant code is available in GLib while compiling.
+ *
+ * The @out_is_markup is always set, providing the information about
+ * used format of the returned string.
+ *
+ * Returns: (transfer full): a new string, containing the @size_bytes formatted as string
+ *
+ * Since: 43
+ **/
+gchar *
+gs_utils_format_size (guint64 size_bytes,
+		      gboolean *out_is_markup)
+{
+#ifdef HAVE_G_FORMAT_SIZE_ONLY_VALUE
+	g_autofree gchar *value_str = g_format_size_full (size_bytes, G_FORMAT_SIZE_ONLY_VALUE);
+	g_autofree gchar *unit_str = g_format_size_full (size_bytes, G_FORMAT_SIZE_ONLY_UNIT);
+	g_autofree gchar *value_escaped = g_markup_escape_text (value_str, -1);
+	g_autofree gchar *unit_escaped = g_markup_printf_escaped ("<span font_size='x-small'>%s</span>", unit_str);
+	g_return_val_if_fail (out_is_markup != NULL, NULL);
+	*out_is_markup = TRUE;
+	/* Translators: This is to construct a disk size string consisting of the value and its unit, while
+	 * the unit is drawn with a smaller font. If you need to flip the order, then you can use "%2$s %1$s".
+	 * Make sure you'll preserve the no break space between the values.
+	 * Example result: "13.0 MB" */
+	return g_strdup_printf (C_("format-size", "%s\302\240%s"), value_escaped, unit_escaped);
+#else /* HAVE_G_FORMAT_SIZE_ONLY_VALUE */
+	g_return_val_if_fail (out_is_markup != NULL, NULL);
+	*out_is_markup = FALSE;
+	return g_format_size (size_bytes);
+#endif /* HAVE_G_FORMAT_SIZE_ONLY_VALUE */
 }
