@@ -16,9 +16,11 @@
 
 G_BEGIN_DECLS
 
-void	 gs_start_spinner		(GtkSpinner	*spinner);
-void	 gs_stop_spinner		(GtkSpinner	*spinner);
-void	 gs_container_remove_all	(GtkContainer	*container);
+typedef void (*GsRemoveFunc) (GtkWidget *container,
+			      GtkWidget *child);
+
+void	 gs_widget_remove_all		(GtkWidget	*container,
+					 GsRemoveFunc    remove_func);
 void	 gs_grab_focus_when_mapped	(GtkWidget	*widget);
 
 void	 gs_app_notify_installed	(GsApp		*app);
@@ -38,6 +40,11 @@ void		 gs_utils_show_error_dialog	(GtkWindow	*parent,
 						 const gchar	*title,
 						 const gchar	*msg,
 						 const gchar	*details);
+gboolean	 gs_utils_ask_user_accepts	(GtkWindow	*parent,
+						 const gchar	*title,
+						 const gchar	*msg,
+						 const gchar	*details,
+						 const gchar	*accept_label);
 gchar		*gs_utils_build_unique_id_kind	(AsComponentKind kind,
 						 const gchar	*id);
 gboolean	 gs_utils_list_has_component_fuzzy	(GsAppList	*list,
@@ -48,6 +55,9 @@ gchar		*gs_utils_time_to_string	(gint64		 unix_time_seconds);
 void		 gs_utils_invoke_reboot_async	(GCancellable	*cancellable,
 						 GAsyncReadyCallback ready_callback,
 						 gpointer	 user_data);
+gboolean	 gs_utils_invoke_reboot_finish	(GObject	*source_object,
+						 GAsyncResult	*result,
+						 GError		**error);
 gboolean	gs_utils_split_time_difference	(gint64 unix_time_seconds,
 						 gint *out_minutes_ago,
 						 gint *out_hours_ago,
@@ -55,22 +65,7 @@ gboolean	gs_utils_split_time_difference	(gint64 unix_time_seconds,
 						 gint *out_weeks_ago,
 						 gint *out_months_ago,
 						 gint *out_years_ago);
-
-#if !GLIB_CHECK_VERSION(2, 62, 0)
-
-#define  g_clear_signal_handler(handler_id_ptr, instance)           \
-  G_STMT_START {                                                    \
-    gpointer const _instance      = (instance);                     \
-    gulong *const _handler_id_ptr = (handler_id_ptr);               \
-    const gulong _handler_id      = *_handler_id_ptr;               \
-                                                                    \
-    if (_handler_id > 0)                                            \
-      {                                                             \
-        *_handler_id_ptr = 0;                                       \
-        g_signal_handler_disconnect (_instance, _handler_id);       \
-      }                                                             \
-  } G_STMT_END                                                      \
-
-#endif
+gchar		*gs_utils_format_size		(guint64 size_bytes,
+						 gboolean *out_is_markup);
 
 G_END_DECLS
