@@ -326,6 +326,12 @@ notify_search_resources (GsExtrasPageMode   mode,
 
 	mode_string = gs_extras_page_mode_to_string (mode);
 
+	/* Make sure non-NULL values are used */
+	if (desktop_id == NULL)
+		desktop_id = "";
+	if (ident == NULL)
+		ident = "";
+
 	n = g_notification_new (title);
 	g_notification_set_body (n, body);
 	/* TRANSLATORS: this is a button that launches gnome-software */
@@ -418,6 +424,12 @@ install_resources (GsExtrasPageMode   mode,
 		                  "&s", &startup_id);
 	}
 
+	/* Make sure non-NULL values are used */
+	if (desktop_id == NULL)
+		desktop_id = "";
+	if (startup_id == NULL)
+		startup_id = "";
+
 	mode_string = gs_extras_page_mode_to_string (mode);
 	g_action_group_activate_action (G_ACTION_GROUP (app), "install-resources",
 	                                g_variant_new ("(s^assss)", mode_string, resources, startup_id, desktop_id, ident ? ident : ""));
