@@ -25,9 +25,7 @@ struct _GsInstalledPage
 
 	GsPluginLoader		*plugin_loader;
 	GCancellable		*cancellable;
-	GtkSizeGroup		*sizegroup_image;
 	GtkSizeGroup		*sizegroup_name;
-	GtkSizeGroup		*sizegroup_desc;
 	GtkSizeGroup		*sizegroup_button_label;
 	GtkSizeGroup		*sizegroup_button_image;
 	gboolean		 cache_valid;
@@ -37,10 +35,17 @@ struct _GsInstalledPage
 	guint			 pending_apps_counter;
 	gboolean		 is_narrow;
 
+	GtkWidget		*group_install_in_progress;
+	GtkWidget		*group_install_apps;
+	GtkWidget		*group_install_system_apps;
+	GtkWidget		*group_install_addons;
+	GtkWidget		*group_install_web_apps;
+
 	GtkWidget		*list_box_install_in_progress;
 	GtkWidget		*list_box_install_apps;
 	GtkWidget		*list_box_install_system_apps;
 	GtkWidget		*list_box_install_addons;
+	GtkWidget		*list_box_install_web_apps;
 	GtkWidget		*scrolledwindow_install;
 	GtkWidget		*spinner_install;
 	GtkWidget		*stack_install;
@@ -57,8 +62,10 @@ typedef enum {
 
 static GParamSpec *obj_props[PROP_IS_NARROW + 1] = { NULL, };
 
-static void gs_installed_page_pending_apps_changed_cb (GsPluginLoader *plugin_loader,
-                                                       GsInstalledPage *self);
+static void gs_installed_page_pending_apps_refined_cb (GObject *source,
+						       GAsyncResult *res,
+						       gpointer user_data);
+static GsPluginRefineFlags gs_installed_page_get_refine_flags (GsInstalledPage *self);
 static void gs_installed_page_notify_state_changed_cb (GsApp *app,
 						       GParamSpec *pspec,
 						       GsInstalledPage *self);
@@ -68,6 +75,7 @@ typedef enum {
 	GS_UPDATE_LIST_SECTION_REMOVABLE_APPS,
 	GS_UPDATE_LIST_SECTION_SYSTEM_APPS,
 	GS_UPDATE_LIST_SECTION_ADDONS,
+	GS_UPDATE_LIST_SECTION_WEB_APPS,
 	GS_UPDATE_LIST_SECTION_LAST
 } GsInstalledPageSection;
 
@@ -84,16 +92,33 @@ gs_installed_page_get_app_section (GsApp *app)
 	    state == GS_APP_STATE_REMOVING)
 		return GS_UPDATE_LIST_SECTION_INSTALLING_AND_REMOVING;
 
-	if (kind == AS_COMPONENT_KIND_DESKTOP_APP ||
-	    kind == AS_COMPONENT_KIND_WEB_APP) {
+	if (kind == AS_COMPONENT_KIND_DESKTOP_APP) {
 		if (gs_app_has_quirk (app, GS_APP_QUIRK_COMPULSORY))
 			return GS_UPDATE_LIST_SECTION_SYSTEM_APPS;
 		return GS_UPDATE_LIST_SECTION_REMOVABLE_APPS;
 	}
 
+	if (kind == AS_COMPONENT_KIND_WEB_APP)
+		return GS_UPDATE_LIST_SECTION_WEB_APPS;
+
 	return GS_UPDATE_LIST_SECTION_ADDONS;
 }
 
+static void
+update_groups (GsInstalledPage *self)
+{
+	gtk_widget_set_visible (self->group_install_in_progress,
+				gtk_widget_get_first_child (self->list_box_install_in_progress) != NULL);
+	gtk_widget_set_visible (self->group_install_apps,
+				gtk_widget_get_first_child (self->list_box_install_apps) != NULL);
+	gtk_widget_set_visible (self->group_install_system_apps,
+				gtk_widget_get_first_child (self->list_box_install_system_apps) != NULL);
+	gtk_widget_set_visible (self->group_install_addons,
+				gtk_widget_get_first_child (self->list_box_install_addons) != NULL);
+	gtk_widget_set_visible (self->group_install_web_apps,
+				gtk_widget_get_first_child (self->list_box_install_web_apps) != NULL);
+}
+
 static GsInstalledPageSection
 gs_installed_page_get_row_section (GsInstalledPage *self,
 				   GsAppRow *app_row)
@@ -112,6 +137,8 @@ gs_installed_page_get_row_section (GsInstalledPage *self,
 		return GS_UPDATE_LIST_SECTION_SYSTEM_APPS;
 	if (parent == self->list_box_install_addons)
 		return GS_UPDATE_LIST_SECTION_ADDONS;
+	if (parent == self->list_box_install_web_apps)
+		return GS_UPDATE_LIST_SECTION_WEB_APPS;
 
 	g_warn_if_reached ();
 
@@ -137,12 +164,15 @@ gs_installed_page_app_row_activated_cb (GtkListBox *list_box,
 static void
 row_unrevealed (GObject *row, GParamSpec *pspec, gpointer data)
 {
+	GsInstalledPage *self = GS_INSTALLED_PAGE (gtk_widget_get_ancestor (GTK_WIDGET (row),
+									    GS_TYPE_INSTALLED_PAGE));
 	GtkWidget *list;
 
 	list = gtk_widget_get_parent (GTK_WIDGET (row));
 	if (list == NULL)
 		return;
-	gtk_container_remove (GTK_CONTAINER (list), GTK_WIDGET (row));
+	gtk_list_box_remove (GTK_LIST_BOX (list), GTK_WIDGET (row));
+	update_groups (self);
 }
 
 static void
@@ -154,17 +184,9 @@ gs_installed_page_unreveal_row (GsAppRow *app_row)
 						      G_CALLBACK (gs_installed_page_notify_state_changed_cb), NULL);
 	}
 
-	/* This check is required, because GsAppRow does not emit
-	 * the signal when the row is not realized. This can happen
-	 * when installing/uninstalling an app without visiting
-	 * the Installed page. */
-	if (!gtk_widget_get_mapped (GTK_WIDGET (app_row))) {
-		row_unrevealed (G_OBJECT (app_row), NULL, NULL);
-	} else {
-		g_signal_connect (app_row, "unrevealed",
-				  G_CALLBACK (row_unrevealed), NULL);
-		gs_app_row_unreveal (app_row);
-	}
+	g_signal_connect (app_row, "unrevealed",
+			  G_CALLBACK (row_unrevealed), NULL);
+	gs_app_row_unreveal (app_row);
 }
 
 static GsAppRow *  /* (transfer none) */
@@ -176,15 +198,15 @@ gs_installed_page_find_app_row (GsInstalledPage *self,
 		self->list_box_install_apps,
 		self->list_box_install_system_apps,
 		self->list_box_install_addons,
+		self->list_box_install_web_apps,
 		NULL
 	};
 
 	for (gsize i = 0; lists[i]; i++) {
-		g_autoptr(GList) children = NULL;
-
-		children = gtk_container_get_children (GTK_CONTAINER (lists[i]));
-		for (GList *l = children; l; l = l->next) {
-			GsAppRow *app_row = GS_APP_ROW (l->data);
+		for (GtkWidget *child = gtk_widget_get_first_child (lists[i]);
+		     child != NULL;
+		     child = gtk_widget_get_next_sibling (child)) {
+			GsAppRow *app_row = GS_APP_ROW (child);
 			if (gs_app_row_get_app (app_row) == app) {
 				return app_row;
 			}
@@ -228,7 +250,7 @@ gs_installed_page_maybe_move_app_row (GsInstalledPage *self,
 		GtkWidget *widget = GTK_WIDGET (app_row);
 
 		g_object_ref (app_row);
-		gtk_container_remove (GTK_CONTAINER (gtk_widget_get_parent (widget)), widget);
+		gtk_list_box_remove (GTK_LIST_BOX (gtk_widget_get_parent (widget)), widget);
 		switch (expected_section) {
 		case GS_UPDATE_LIST_SECTION_INSTALLING_AND_REMOVING:
 			widget = self->list_box_install_in_progress;
@@ -242,6 +264,9 @@ gs_installed_page_maybe_move_app_row (GsInstalledPage *self,
 		case GS_UPDATE_LIST_SECTION_ADDONS:
 			widget = self->list_box_install_addons;
 			break;
+		case GS_UPDATE_LIST_SECTION_WEB_APPS:
+			widget = self->list_box_install_web_apps;
+			break;
 		default:
 			g_warn_if_reached ();
 			widget = NULL;
@@ -249,9 +274,10 @@ gs_installed_page_maybe_move_app_row (GsInstalledPage *self,
 		}
 
 		if (widget != NULL)
-			gtk_container_add (GTK_CONTAINER (widget), GTK_WIDGET (app_row));
+			gtk_list_box_append (GTK_LIST_BOX (widget), GTK_WIDGET (app_row));
 
 		g_object_unref (app_row);
+		update_groups (self);
 	}
 }
 
@@ -290,9 +316,15 @@ gs_installed_page_is_actual_app (GsApp *app)
 {
 	if (gs_app_get_description (app) != NULL)
 		return TRUE;
+
 	/* special snowflake */
 	if (g_strcmp0 (gs_app_get_id (app), "google-chrome.desktop") == 0)
 		return TRUE;
+
+	/* web apps sometimes don't have descriptions */
+	if (gs_app_get_kind (app) == AS_COMPONENT_KIND_WEB_APP)
+		return TRUE;
+
 	g_debug ("%s is not an actual app", gs_app_get_unique_id (app));
 	return FALSE;
 }
@@ -321,25 +353,28 @@ gs_installed_page_add_app (GsInstalledPage *self, GsAppList *list, GsApp *app)
 
 	switch (gs_installed_page_get_app_section (app)) {
 	case GS_UPDATE_LIST_SECTION_INSTALLING_AND_REMOVING:
-		gtk_container_add (GTK_CONTAINER (self->list_box_install_in_progress), app_row);
+		gtk_list_box_append (GTK_LIST_BOX (self->list_box_install_in_progress), app_row);
 		break;
 	case GS_UPDATE_LIST_SECTION_REMOVABLE_APPS:
-		gtk_container_add (GTK_CONTAINER (self->list_box_install_apps), app_row);
+		gtk_list_box_append (GTK_LIST_BOX (self->list_box_install_apps), app_row);
 		break;
 	case GS_UPDATE_LIST_SECTION_SYSTEM_APPS:
-		gtk_container_add (GTK_CONTAINER (self->list_box_install_system_apps), app_row);
+		gtk_list_box_append (GTK_LIST_BOX (self->list_box_install_system_apps), app_row);
 		break;
 	case GS_UPDATE_LIST_SECTION_ADDONS:
-		gtk_container_add (GTK_CONTAINER (self->list_box_install_addons), app_row);
+		gtk_list_box_append (GTK_LIST_BOX (self->list_box_install_addons), app_row);
+		break;
+	case GS_UPDATE_LIST_SECTION_WEB_APPS:
+		gtk_list_box_append (GTK_LIST_BOX (self->list_box_install_web_apps), app_row);
 		break;
 	default:
 		g_assert_not_reached ();
 	}
 
+	update_groups (self);
+
 	gs_app_row_set_size_groups (GS_APP_ROW (app_row),
-				    self->sizegroup_image,
 				    self->sizegroup_name,
-				    self->sizegroup_desc,
 				    self->sizegroup_button_label,
 				    self->sizegroup_button_image);
 
@@ -359,8 +394,10 @@ gs_installed_page_get_installed_cb (GObject *source_object,
 	GsPluginLoader *plugin_loader = GS_PLUGIN_LOADER (source_object);
 	g_autoptr(GError) error = NULL;
 	g_autoptr(GsAppList) list = NULL;
+	g_autoptr(GsAppList) pending = gs_plugin_loader_get_pending (plugin_loader);
+	g_autoptr(GsPluginJob) plugin_job = NULL;
 
-	gs_stop_spinner (GTK_SPINNER (self->spinner_install));
+	gtk_spinner_stop (GTK_SPINNER (self->spinner_install));
 	gtk_stack_set_visible_child_name (GTK_STACK (self->stack_install), "view");
 
 	self->waiting = FALSE;
@@ -370,7 +407,8 @@ gs_installed_page_get_installed_cb (GObject *source_object,
 						    res,
 						    &error);
 	if (list == NULL) {
-		if (!g_error_matches (error, GS_PLUGIN_ERROR, GS_PLUGIN_ERROR_CANCELLED))
+		if (!g_error_matches (error, GS_PLUGIN_ERROR, GS_PLUGIN_ERROR_CANCELLED) &&
+		    !g_error_matches (error, G_IO_ERROR, G_IO_ERROR_CANCELLED))
 			g_warning ("failed to get installed apps: %s", error->message);
 		goto out;
 	}
@@ -379,15 +417,21 @@ gs_installed_page_get_installed_cb (GObject *source_object,
 		gs_installed_page_add_app (self, list, app);
 	}
 out:
-	gs_installed_page_pending_apps_changed_cb (plugin_loader, self);
+	if (gs_app_list_length (pending) > 0) {
+		plugin_job = gs_plugin_job_refine_new (pending,
+						       gs_installed_page_get_refine_flags (self));
+		gs_plugin_loader_job_process_async (self->plugin_loader, plugin_job,
+						    self->cancellable,
+						    gs_installed_page_pending_apps_refined_cb,
+						    self);
+
+	}
 }
 
 static void
-gs_installed_page_remove_all_cb (GtkWidget *child,
-				 gpointer user_data)
+gs_installed_page_remove_all_cb (GtkWidget *container,
+				 GtkWidget *child)
 {
-	GtkContainer *container = user_data;
-
 	if (GS_IS_APP_ROW (child)) {
 		GsApp *app = gs_app_row_get_app (GS_APP_ROW (child));
 		if (app != NULL) {
@@ -398,31 +442,34 @@ gs_installed_page_remove_all_cb (GtkWidget *child,
 		g_warn_if_reached ();
 	}
 
-	gtk_container_remove (container, child);
+	gtk_list_box_remove (GTK_LIST_BOX (container), child);
 }
 
-static void
-gs_container_remove_all_with_cb (GtkContainer *container,
-				 GtkCallback callback)
+static gboolean
+filter_app_kinds_cb (GsApp    *app,
+                     gpointer  user_data)
 {
-	gtk_container_foreach (container, callback, container);
+	/* Remove invalid apps. */
+	if (!gs_plugin_loader_app_is_valid (app, GS_PLUGIN_REFINE_FLAGS_NONE))
+		return FALSE;
+
+	switch (gs_app_get_kind (app)) {
+	case AS_COMPONENT_KIND_OPERATING_SYSTEM:
+	case AS_COMPONENT_KIND_CODEC:
+	case AS_COMPONENT_KIND_FONT:
+		g_debug ("app invalid as %s: %s",
+			 as_component_kind_to_string (gs_app_get_kind (app)),
+			 gs_app_get_unique_id (app));
+		return FALSE;
+	default:
+		return TRUE;
+	}
 }
 
-static void
-gs_installed_page_load (GsInstalledPage *self)
+static GsPluginRefineFlags
+gs_installed_page_get_refine_flags (GsInstalledPage *self)
 {
 	GsPluginRefineFlags flags;
-	g_autoptr(GsPluginJob) plugin_job = NULL;
-
-	if (self->waiting)
-		return;
-	self->waiting = TRUE;
-
-	/* remove old entries */
-	gs_container_remove_all_with_cb (GTK_CONTAINER (self->list_box_install_in_progress), gs_installed_page_remove_all_cb);
-	gs_container_remove_all_with_cb (GTK_CONTAINER (self->list_box_install_apps), gs_installed_page_remove_all_cb);
-	gs_container_remove_all_with_cb (GTK_CONTAINER (self->list_box_install_system_apps), gs_installed_page_remove_all_cb);
-	gs_container_remove_all_with_cb (GTK_CONTAINER (self->list_box_install_addons), gs_installed_page_remove_all_cb);
 
 	flags = GS_PLUGIN_REFINE_FLAGS_REQUIRE_ICON |
 		GS_PLUGIN_REFINE_FLAGS_REQUIRE_HISTORY |
@@ -439,17 +486,39 @@ gs_installed_page_load (GsInstalledPage *self)
 	if (should_show_installed_size (self))
 		flags |= GS_PLUGIN_REFINE_FLAGS_REQUIRE_SIZE;
 
+	return flags;
+}
+
+static void
+gs_installed_page_load (GsInstalledPage *self)
+{
+	g_autoptr(GsAppQuery) query = NULL;
+	g_autoptr(GsPluginJob) plugin_job = NULL;
+
+	if (self->waiting)
+		return;
+	self->waiting = TRUE;
+
+	/* remove old entries */
+	gs_widget_remove_all (self->list_box_install_in_progress, gs_installed_page_remove_all_cb);
+	gs_widget_remove_all (self->list_box_install_apps, gs_installed_page_remove_all_cb);
+	gs_widget_remove_all (self->list_box_install_system_apps, gs_installed_page_remove_all_cb);
+	gs_widget_remove_all (self->list_box_install_addons, gs_installed_page_remove_all_cb);
+	gs_widget_remove_all (self->list_box_install_web_apps, gs_installed_page_remove_all_cb);
+	update_groups (self);
+
 	/* get installed apps */
-	plugin_job = gs_plugin_job_newv (GS_PLUGIN_ACTION_GET_INSTALLED,
-					 "refine-flags", flags,
-					 "dedupe-flags", GS_APP_LIST_FILTER_FLAG_NONE,
-					 NULL);
+	query = gs_app_query_new ("is-installed", GS_APP_QUERY_TRISTATE_TRUE,
+				  "refine-flags", gs_installed_page_get_refine_flags (self),
+				  "filter-func", filter_app_kinds_cb,
+				  NULL);
+	plugin_job = gs_plugin_job_list_apps_new (query, GS_PLUGIN_LIST_APPS_FLAGS_INTERACTIVE);
 	gs_plugin_loader_job_process_async (self->plugin_loader,
 					    plugin_job,
 					    self->cancellable,
 					    gs_installed_page_get_installed_cb,
 					    self);
-	gs_start_spinner (GTK_SPINNER (self->spinner_install));
+	gtk_spinner_start (GTK_SPINNER (self->spinner_install));
 	gtk_stack_set_visible_child_name (GTK_STACK (self->stack_install), "spinner");
 }
 
@@ -576,12 +645,6 @@ gs_installed_page_sort_func (GtkListBoxRow *a,
 	g_autofree gchar *key1 = NULL;
 	g_autofree gchar *key2 = NULL;
 
-	/* check valid */
-	if (!GTK_IS_BIN(a) || !GTK_IS_BIN(b)) {
-		g_warning ("GtkListBoxRow not valid");
-		return 0;
-	}
-
 	a1 = gs_app_row_get_app (GS_APP_ROW (a));
 	a2 = gs_app_row_get_app (GS_APP_ROW (b));
 	key1 = gs_installed_page_get_app_sort_key (a1);
@@ -600,15 +663,15 @@ gs_installed_page_has_app (GsInstalledPage *self,
 		self->list_box_install_apps,
 		self->list_box_install_system_apps,
 		self->list_box_install_addons,
+		self->list_box_install_web_apps,
 		NULL
 	};
 
 	for (gsize i = 0; lists[i]; i++) {
-		g_autoptr(GList) children = NULL;
-
-		children = gtk_container_get_children (GTK_CONTAINER (lists[i]));
-		for (GList *l = children; l; l = l->next) {
-			GsAppRow *app_row = GS_APP_ROW (l->data);
+		for (GtkWidget *child = gtk_widget_get_first_child (lists[i]);
+		     child != NULL;
+		     child = gtk_widget_get_next_sibling (child)) {
+			GsAppRow *app_row = GS_APP_ROW (child);
 			if (gs_app_row_get_app (app_row) == app)
 				return TRUE;
 		}
@@ -618,39 +681,79 @@ gs_installed_page_has_app (GsInstalledPage *self,
 }
 
 static void
-gs_installed_page_pending_apps_changed_cb (GsPluginLoader *plugin_loader,
-                                           GsInstalledPage *self)
+gs_installed_page_add_pending_apps (GsInstalledPage *self,
+				    GsAppList *list,
+				    gboolean should_install)
 {
-	GsApp *app;
-	guint i;
-	guint cnt = 0;
-	g_autoptr(GsAppList) pending = NULL;
+	guint pending_apps_count = 0;
 
-	/* add new apps to the list */
-	pending = gs_plugin_loader_get_pending (plugin_loader);
-	for (i = 0; i < gs_app_list_length (pending); i++) {
-		app = gs_app_list_index (pending, i);
+	for (guint i = 0; i < gs_app_list_length (list); ++i) {
+		GsApp *app = gs_app_list_index (list, i);
+		if (gs_app_is_installed (app)) {
+			continue;
+		}
 
 		/* never show OS upgrades, we handle the scheduling and
 		 * cancellation in GsUpgradeBanner */
 		if (gs_app_get_kind (app) == AS_COMPONENT_KIND_OPERATING_SYSTEM)
 			continue;
 
-		/* do not to add pending apps more than once. */
-		if (gs_installed_page_has_app (self, app) == FALSE)
-			gs_installed_page_add_app (self, pending, app);
+		if (gs_app_get_state (app) == GS_APP_STATE_AVAILABLE)
+			gs_app_set_state (app, GS_APP_STATE_QUEUED_FOR_INSTALL);
 
-		/* incremement the label */
-		cnt++;
+		if (should_install &&
+		    gs_app_get_state (app) == GS_APP_STATE_QUEUED_FOR_INSTALL &&
+		    gs_plugin_loader_get_network_available (self->plugin_loader) &&
+		    !gs_plugin_loader_get_network_metered (self->plugin_loader))
+			gs_page_install_app (GS_PAGE (self), app,
+					     GS_SHELL_INTERACTION_FULL,
+					     gs_app_get_cancellable (app));
+
+		++pending_apps_count;
+		if (!gs_installed_page_has_app (self, app))
+			gs_installed_page_add_app (self, list, app);
 	}
 
 	/* update the number of on-going operations */
-	if (cnt != self->pending_apps_counter) {
-		self->pending_apps_counter = cnt;
+	if (pending_apps_count != self->pending_apps_counter) {
+		self->pending_apps_counter = pending_apps_count;
 		g_object_notify (G_OBJECT (self), "counter");
 	}
 }
 
+static void
+gs_installed_page_pending_apps_refined_cb (GObject *source,
+					   GAsyncResult *res,
+					   gpointer user_data)
+{
+	GsPluginLoader *plugin_loader = GS_PLUGIN_LOADER (source);
+	GsInstalledPage *self = GS_INSTALLED_PAGE (user_data);
+	g_autoptr(GError) error = NULL;
+	g_autoptr(GsAppList) list = gs_plugin_loader_job_process_finish (plugin_loader,
+									 res,
+									 &error);
+	if (list == NULL) {
+		if (!g_error_matches (error, GS_PLUGIN_ERROR, GS_PLUGIN_ERROR_CANCELLED))
+			g_warning ("failed to refine pending apps: %s", error->message);
+		return;
+	}
+
+	/* we add the pending apps and install them because this is called after we
+	 * populate the page, and there may be pending apps coming from the saved list
+	 * (i.e. after loading the saved pending apps from the disk) */
+	gs_installed_page_add_pending_apps (self, list, TRUE);
+}
+
+static void
+gs_installed_page_pending_apps_changed_cb (GsPluginLoader *plugin_loader,
+                                           GsInstalledPage *self)
+{
+	g_autoptr(GsAppList) pending = gs_plugin_loader_get_pending (plugin_loader);
+	/* we don't call install every time the pending apps list changes because
+	 * it may be queued in the plugin loader */
+	gs_installed_page_add_pending_apps (self, pending, FALSE);
+}
+
 static gboolean
 gs_installed_page_setup (GsPage *page,
                          GsShell *shell,
@@ -683,6 +786,9 @@ gs_installed_page_setup (GsPage *page,
 	gtk_list_box_set_sort_func (GTK_LIST_BOX (self->list_box_install_addons),
 				    gs_installed_page_sort_func,
 				    self, NULL);
+	gtk_list_box_set_sort_func (GTK_LIST_BOX (self->list_box_install_web_apps),
+				    gs_installed_page_sort_func,
+				    self, NULL);
 	return TRUE;
 }
 
@@ -703,7 +809,7 @@ gs_installed_page_get_property (GObject    *object,
 		break;
 	case PROP_TITLE:
 		/* Translators: This is in the context of a list of apps which are installed on the system. */
-		g_value_set_string (value, _("Installed"));
+		g_value_set_string (value, C_("List of installed apps", "Installed"));
 		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
@@ -739,9 +845,7 @@ gs_installed_page_dispose (GObject *object)
 {
 	GsInstalledPage *self = GS_INSTALLED_PAGE (object);
 
-	g_clear_object (&self->sizegroup_image);
 	g_clear_object (&self->sizegroup_name);
-	g_clear_object (&self->sizegroup_desc);
 	g_clear_object (&self->sizegroup_button_label);
 	g_clear_object (&self->sizegroup_button_image);
 
@@ -752,17 +856,6 @@ gs_installed_page_dispose (GObject *object)
 	G_OBJECT_CLASS (gs_installed_page_parent_class)->dispose (object);
 }
 
-static void
-update_group_visibility_cb (GtkWidget *group,
-			    GtkWidget *widget,
-			    GtkWidget *list_box)
-{
-	g_autoptr(GList) children = NULL;
-
-	children = gtk_container_get_children (GTK_CONTAINER (list_box));
-	gtk_widget_set_visible (group, children != NULL);
-}
-
 static void
 gs_installed_page_class_init (GsInstalledPageClass *klass)
 {
@@ -802,16 +895,21 @@ gs_installed_page_class_init (GsInstalledPageClass *klass)
 
 	gtk_widget_class_set_template_from_resource (widget_class, "/org/gnome/Software/gs-installed-page.ui");
 
+	gtk_widget_class_bind_template_child (widget_class, GsInstalledPage, group_install_in_progress);
+	gtk_widget_class_bind_template_child (widget_class, GsInstalledPage, group_install_apps);
+	gtk_widget_class_bind_template_child (widget_class, GsInstalledPage, group_install_system_apps);
+	gtk_widget_class_bind_template_child (widget_class, GsInstalledPage, group_install_addons);
+	gtk_widget_class_bind_template_child (widget_class, GsInstalledPage, group_install_web_apps);
 	gtk_widget_class_bind_template_child (widget_class, GsInstalledPage, list_box_install_in_progress);
 	gtk_widget_class_bind_template_child (widget_class, GsInstalledPage, list_box_install_apps);
 	gtk_widget_class_bind_template_child (widget_class, GsInstalledPage, list_box_install_system_apps);
 	gtk_widget_class_bind_template_child (widget_class, GsInstalledPage, list_box_install_addons);
+	gtk_widget_class_bind_template_child (widget_class, GsInstalledPage, list_box_install_web_apps);
 	gtk_widget_class_bind_template_child (widget_class, GsInstalledPage, scrolledwindow_install);
 	gtk_widget_class_bind_template_child (widget_class, GsInstalledPage, spinner_install);
 	gtk_widget_class_bind_template_child (widget_class, GsInstalledPage, stack_install);
 
 	gtk_widget_class_bind_template_callback (widget_class, gs_installed_page_app_row_activated_cb);
-	gtk_widget_class_bind_template_callback (widget_class, update_group_visibility_cb);
 }
 
 static void
@@ -819,9 +917,7 @@ gs_installed_page_init (GsInstalledPage *self)
 {
 	gtk_widget_init_template (GTK_WIDGET (self));
 
-	self->sizegroup_image = gtk_size_group_new (GTK_SIZE_GROUP_HORIZONTAL);
 	self->sizegroup_name = gtk_size_group_new (GTK_SIZE_GROUP_HORIZONTAL);
-	self->sizegroup_desc = gtk_size_group_new (GTK_SIZE_GROUP_HORIZONTAL);
 	self->sizegroup_button_label = gtk_size_group_new (GTK_SIZE_GROUP_HORIZONTAL);
 	self->sizegroup_button_image = gtk_size_group_new (GTK_SIZE_GROUP_HORIZONTAL);
 
