@@ -12,11 +12,13 @@
 #include <glib-object.h>
 #include <gtk/gtk.h>
 
+#include "gs-info-window.h"
+
 G_BEGIN_DECLS
 
 #define GS_TYPE_METERED_DATA_DIALOG (gs_metered_data_dialog_get_type ())
 
-G_DECLARE_FINAL_TYPE (GsMeteredDataDialog, gs_metered_data_dialog, GS, METERED_DATA_DIALOG, GtkDialog)
+G_DECLARE_FINAL_TYPE (GsMeteredDataDialog, gs_metered_data_dialog, GS, METERED_DATA_DIALOG, GsInfoWindow)
 
 GtkWidget	*gs_metered_data_dialog_new	(GtkWindow	*parent);
 
