@@ -8,8 +8,8 @@
 
 #pragma once
 
+#include <adwaita.h>
 #include <gtk/gtk.h>
-#include <handy.h>
 
 #include "gnome-software-private.h"
 #include "gs-app.h"
@@ -18,10 +18,9 @@ G_BEGIN_DECLS
 
 #define GS_TYPE_REPOS_SECTION (gs_repos_section_get_type ())
 
-G_DECLARE_FINAL_TYPE (GsReposSection, gs_repos_section, GS, REPOS_SECTION, HdyPreferencesGroup)
+G_DECLARE_FINAL_TYPE (GsReposSection, gs_repos_section, GS, REPOS_SECTION, AdwPreferencesGroup)
 
-GtkWidget	*gs_repos_section_new			(GsPluginLoader		*plugin_loader,
-							 gboolean		 always_allow_enable_disable);
+GtkWidget	*gs_repos_section_new			(gboolean		 always_allow_enable_disable);
 void		 gs_repos_section_add_repo		(GsReposSection		*self,
 							 GsApp			*repo);
 const gchar	*gs_repos_section_get_title		(GsReposSection		*self);
