@@ -14,13 +14,12 @@
 
 #include "gs-screenshot-image.h"
 #include "gs-common.h"
-#include "gs-picture.h"
 
 #define SPINNER_TIMEOUT_SECS 2
 
 struct _GsScreenshotImage
 {
-	GtkBin		 parent_instance;
+	GtkWidget	 parent_instance;
 
 	AsScreenshot	*screenshot;
 	GtkWidget	*spinner;
@@ -28,10 +27,12 @@ struct _GsScreenshotImage
 	GtkWidget	*box_error;
 	GtkWidget	*image1;
 	GtkWidget	*image2;
+	GtkWidget	*video;
 	GtkWidget	*label_error;
 	GSettings	*settings;
 	SoupSession	*session;
 	SoupMessage	*message;
+	GCancellable	*cancellable;
 	gchar		*filename;
 	const gchar	*current_image;
 	guint		 width;
@@ -41,7 +42,26 @@ struct _GsScreenshotImage
 	gboolean	 showing_image;
 };
 
-G_DEFINE_TYPE (GsScreenshotImage, gs_screenshot_image, GTK_TYPE_BIN)
+G_DEFINE_TYPE (GsScreenshotImage, gs_screenshot_image, GTK_TYPE_WIDGET)
+
+enum {
+	SIGNAL_CLICKED,
+	SIGNAL_LAST
+};
+
+static guint signals [SIGNAL_LAST] = { 0 };
+
+static void
+gs_screenshot_image_clicked_cb (GtkGestureClick *gesture,
+				gint n_press,
+				gdouble x,
+				gdouble y,
+				gpointer user_data)
+{
+	GsScreenshotImage *self = user_data;
+	if (n_press == 1)
+		g_signal_emit (self, signals[SIGNAL_CLICKED], 0);
+}
 
 AsScreenshot *
 gs_screenshot_image_get_screenshot (GsScreenshotImage *ssimg)
@@ -54,13 +74,13 @@ static void
 gs_screenshot_image_start_spinner (GsScreenshotImage *ssimg)
 {
 	gtk_widget_show (ssimg->spinner);
-	gs_start_spinner (GTK_SPINNER (ssimg->spinner));
+	gtk_spinner_start (GTK_SPINNER (ssimg->spinner));
 }
 
 static void
 gs_screenshot_image_stop_spinner (GsScreenshotImage *ssimg)
 {
-	gs_stop_spinner (GTK_SPINNER (ssimg->spinner));
+	gtk_spinner_stop (GTK_SPINNER (ssimg->spinner));
 	gtk_widget_hide (ssimg->spinner);
 }
 
@@ -83,33 +103,36 @@ gs_screenshot_image_set_error (GsScreenshotImage *ssimg, const gchar *message)
 static void
 as_screenshot_show_image (GsScreenshotImage *ssimg)
 {
-	g_autoptr(GdkPixbuf) pixbuf = NULL;
-
-	/* no need to composite */
-	if (ssimg->width == G_MAXUINT || ssimg->height == G_MAXUINT) {
-		pixbuf = gdk_pixbuf_new_from_file (ssimg->filename, NULL);
+	if (as_screenshot_get_media_kind (ssimg->screenshot) == AS_SCREENSHOT_MEDIA_KIND_VIDEO) {
+		gtk_video_set_filename (GTK_VIDEO (ssimg->video), ssimg->filename);
+		ssimg->current_image = "video";
 	} else {
-		/* this is always going to have alpha */
-		pixbuf = gdk_pixbuf_new_from_file_at_scale (ssimg->filename,
-							    (gint) (ssimg->width * ssimg->scale),
-							    (gint) (ssimg->height * ssimg->scale),
-							    FALSE, NULL);
+		g_autoptr(GdkPixbuf) pixbuf = NULL;
+
+		/* no need to composite */
+		if (ssimg->width == G_MAXUINT || ssimg->height == G_MAXUINT) {
+			pixbuf = gdk_pixbuf_new_from_file (ssimg->filename, NULL);
+		} else {
+			/* this is always going to have alpha */
+			pixbuf = gdk_pixbuf_new_from_file_at_scale (ssimg->filename,
+								    (gint) (ssimg->width * ssimg->scale),
+								    (gint) (ssimg->height * ssimg->scale),
+								    FALSE, NULL);
+		}
+
+		/* show icon */
+		if (g_strcmp0 (ssimg->current_image, "image1") == 0) {
+			if (pixbuf != NULL)
+				gtk_picture_set_pixbuf (GTK_PICTURE (ssimg->image2), pixbuf);
+			ssimg->current_image = "image2";
+		} else {
+			if (pixbuf != NULL)
+				gtk_picture_set_pixbuf (GTK_PICTURE (ssimg->image1), pixbuf);
+			ssimg->current_image = "image1";
+		}
 	}
 
-	/* show icon */
-	if (g_strcmp0 (ssimg->current_image, "image1") == 0) {
-		if (pixbuf != NULL) {
-			gs_picture_set_pixbuf (GS_PICTURE (ssimg->image2), pixbuf);
-		}
-		gtk_stack_set_visible_child_name (GTK_STACK (ssimg->stack), "image2");
-		ssimg->current_image = "image2";
-	} else {
-		if (pixbuf != NULL) {
-			gs_picture_set_pixbuf (GS_PICTURE (ssimg->image1), pixbuf);
-		}
-		gtk_stack_set_visible_child_name (GTK_STACK (ssimg->stack), "image1");
-		ssimg->current_image = "image1";
-	}
+	gtk_stack_set_visible_child_name (GTK_STACK (ssimg->stack), ssimg->current_image);
 
 	gtk_widget_show (GTK_WIDGET (ssimg));
 	ssimg->showing_image = TRUE;
@@ -222,10 +245,15 @@ gs_screenshot_image_show_blurred (GsScreenshotImage *ssimg,
 	if (pb == NULL)
 		return;
 
+	if (g_strcmp0 (ssimg->current_image, "video") == 0) {
+		ssimg->current_image = "image1";
+		gtk_stack_set_visible_child_name (GTK_STACK (ssimg->stack), ssimg->current_image);
+	}
+
 	if (g_strcmp0 (ssimg->current_image, "image1") == 0) {
-		gs_picture_set_pixbuf (GS_PICTURE (ssimg->image1), pb);
+		gtk_picture_set_pixbuf (GTK_PICTURE (ssimg->image1), pb);
 	} else {
-		gs_picture_set_pixbuf (GS_PICTURE (ssimg->image2), pb);
+		gtk_picture_set_pixbuf (GTK_PICTURE (ssimg->image2), pb);
 	}
 }
 
@@ -304,37 +332,83 @@ gs_screenshot_image_save_downloaded_img (GsScreenshotImage *ssimg,
 }
 
 static void
+#if SOUP_CHECK_VERSION(3, 0, 0)
+gs_screenshot_image_complete_cb (GObject *source_object,
+				 GAsyncResult *result,
+				 gpointer user_data)
+#else
 gs_screenshot_image_complete_cb (SoupSession *session,
 				 SoupMessage *msg,
 				 gpointer user_data)
+#endif
 {
 	g_autoptr(GsScreenshotImage) ssimg = GS_SCREENSHOT_IMAGE (user_data);
 	gboolean ret;
 	g_autoptr(GError) error = NULL;
 	g_autoptr(GdkPixbuf) pixbuf = NULL;
 	g_autoptr(GInputStream) stream = NULL;
+	guint status_code;
 
+#if SOUP_CHECK_VERSION(3, 0, 0)
+	SoupMessage *msg;
+
+	stream = soup_session_send_finish (SOUP_SESSION (source_object), result, &error);
+	if (stream == NULL) {
+		if (!g_error_matches (error, G_IO_ERROR, G_IO_ERROR_CANCELLED)) {
+			g_warning ("Failed to download screenshot: %s", error->message);
+			/* Reset the width request, thus the image shrinks when the window width is small */
+			gtk_widget_set_size_request (ssimg->stack, -1, (gint) ssimg->height);
+			gs_screenshot_image_stop_spinner (ssimg);
+			gs_screenshot_image_set_error (ssimg, _("Screenshot not found"));
+		}
+		return;
+	}
+
+	msg = soup_session_get_async_result_message (SOUP_SESSION (source_object), result);
+	status_code = soup_message_get_status (msg);
+#else
+	status_code = msg->status_code;
+#endif
 	if (ssimg->load_timeout_id) {
 		g_source_remove (ssimg->load_timeout_id);
 		ssimg->load_timeout_id = 0;
 	}
 
 	/* return immediately if the message was cancelled or if we're in destruction */
-	if (msg->status_code == SOUP_STATUS_CANCELLED || ssimg->session == NULL)
+#if SOUP_CHECK_VERSION(3, 0, 0)
+	if (ssimg->session == NULL)
+#else
+	if (status_code == SOUP_STATUS_CANCELLED || ssimg->session == NULL)
+#endif
 		return;
 
-	if (msg->status_code == SOUP_STATUS_NOT_MODIFIED) {
+	/* Reset the width request, thus the image shrinks when the window width is small */
+	gtk_widget_set_size_request (ssimg->stack, -1, (gint) ssimg->height);
+
+	if (status_code == SOUP_STATUS_NOT_MODIFIED) {
 		g_debug ("screenshot has not been modified");
 		as_screenshot_show_image (ssimg);
 		gs_screenshot_image_stop_spinner (ssimg);
 		return;
 	}
-	if (msg->status_code != SOUP_STATUS_OK) {
+	if (status_code != SOUP_STATUS_OK) {
 		/* Ignore failures due to being offline */
-		if (msg->status_code != SOUP_STATUS_CANT_RESOLVE)
+#if SOUP_CHECK_VERSION(3, 0, 0)
+		if (!g_error_matches (error, G_IO_ERROR, G_IO_ERROR_HOST_UNREACHABLE) &&
+		    !g_error_matches (error, G_IO_ERROR, G_IO_ERROR_NETWORK_UNREACHABLE)) {
+#else
+		if (status_code != SOUP_STATUS_CANT_RESOLVE) {
+#endif
+			const gchar *reason_phrase;
+#if SOUP_CHECK_VERSION(3, 0, 0)
+			reason_phrase = soup_message_get_reason_phrase (msg);
+#else
+			reason_phrase = msg->reason_phrase;
+#endif
 			g_warning ("Result of screenshot downloading attempt with "
-				   "status code '%u': %s", msg->status_code,
-				   msg->reason_phrase);
+				   "status code '%u': %s", status_code,
+				   reason_phrase);
+		}
 		gs_screenshot_image_stop_spinner (ssimg);
 		/* if we're already showing an image, then don't set the error
 		 * as having an image (even if outdated) is better */
@@ -346,6 +420,7 @@ gs_screenshot_image_complete_cb (SoupSession *session,
 		return;
 	}
 
+#if !SOUP_CHECK_VERSION(3, 0, 0)
 	/* create a buffer with the data */
 	stream = g_memory_input_stream_new_from_data (msg->response_body->data,
 						      msg->response_body->length,
@@ -354,6 +429,7 @@ gs_screenshot_image_complete_cb (SoupSession *session,
 		gs_screenshot_image_stop_spinner (ssimg);
 		return;
 	}
+#endif
 
 	/* load the image */
 	pixbuf = gdk_pixbuf_new_from_stream (stream, NULL, NULL);
@@ -367,10 +443,10 @@ gs_screenshot_image_complete_cb (SoupSession *session,
 	if (ssimg->width == G_MAXUINT || ssimg->height == G_MAXUINT ||
 	    (ssimg->width * ssimg->scale == (guint) gdk_pixbuf_get_width (pixbuf) &&
 	     ssimg->height * ssimg->scale == (guint) gdk_pixbuf_get_height (pixbuf))) {
-		ret = g_file_set_contents (ssimg->filename,
-					   msg->response_body->data,
-					   msg->response_body->length,
-					   &error);
+		ret = gs_pixbuf_save_filename (pixbuf, ssimg->filename,
+					       gdk_pixbuf_get_width (pixbuf),
+					       gdk_pixbuf_get_height (pixbuf),
+					       &error);
 		if (!ret) {
 			gs_screenshot_image_set_error (ssimg, error->message);
 			return;
@@ -414,6 +490,7 @@ gs_screenshot_image_set_size (GsScreenshotImage *ssimg,
 
 	ssimg->width = width;
 	ssimg->height = height;
+	/* Reset the width request, thus the image shrinks when the window width is small */
 	gtk_widget_set_size_request (ssimg->stack, -1, (gint) height);
 }
 
@@ -451,7 +528,12 @@ gs_screenshot_soup_msg_set_modified_request (SoupMessage *msg, GFile *file)
 	date_time = g_date_time_new_from_timeval_local (&time_val);
 #endif
 	mod_date = g_date_time_format (date_time, "%a, %d %b %Y %H:%M:%S %Z");
-	soup_message_headers_append (msg->request_headers,
+	soup_message_headers_append (
+#if SOUP_CHECK_VERSION(3, 0, 0)
+				     soup_message_get_request_headers (msg),
+#else
+				     msg->request_headers,
+#endif
 				     "If-Modified-Since",
 				     mod_date);
 }
@@ -467,17 +549,91 @@ gs_screenshot_show_spinner_cb (gpointer user_data)
 	return FALSE;
 }
 
+static const gchar *
+gs_screenshot_image_get_url (GsScreenshotImage *ssimg)
+{
+	const gchar *url = NULL;
+
+	/* load an image according to the scale factor */
+	ssimg->scale = (guint) gtk_widget_get_scale_factor (GTK_WIDGET (ssimg));
+
+	if (as_screenshot_get_media_kind (ssimg->screenshot) == AS_SCREENSHOT_MEDIA_KIND_VIDEO) {
+		GPtrArray *videos;
+		AsVideo *best_video = NULL;
+		gint64 best_size = G_MAXINT64;
+		gint64 wh = (gint64) ssimg->width * ssimg->scale * ssimg->height * ssimg->scale;
+
+		videos = as_screenshot_get_videos (ssimg->screenshot);
+		for (guint i = 0; videos != NULL && i < videos->len; i++) {
+			AsVideo *adept = g_ptr_array_index (videos, i);
+			gint64 tmp;
+
+			tmp = ABS (wh - (gint64) (as_video_get_width (adept) * as_video_get_height (adept)));
+			if (tmp < best_size) {
+				best_size = tmp;
+				best_video = adept;
+				if (!tmp)
+					break;
+			}
+		}
+
+		if (best_video)
+			url = as_video_get_url (best_video);
+	} else if (as_screenshot_get_media_kind (ssimg->screenshot) == AS_SCREENSHOT_MEDIA_KIND_IMAGE) {
+		AsImage *im;
+
+		im = as_screenshot_get_image (ssimg->screenshot,
+					      ssimg->width * ssimg->scale,
+					      ssimg->height * ssimg->scale);
+
+		/* if we've failed to load a HiDPI image, fallback to LoDPI */
+		if (im == NULL && ssimg->scale > 1) {
+			ssimg->scale = 1;
+			im = as_screenshot_get_image (ssimg->screenshot,
+						      ssimg->width,
+						      ssimg->height);
+		}
+
+		if (im)
+			url = as_image_get_url (im);
+	}
+
+	return url;
+}
+
+static void
+gs_screenshot_video_downloaded_cb (GObject *source_object,
+				   GAsyncResult *result,
+				   gpointer user_data)
+{
+	g_autoptr(GsScreenshotImage) ssimg = user_data;
+	g_autoptr(GError) error = NULL;
+
+	if (gs_download_file_finish (ssimg->session, result, &error) ||
+	    g_error_matches (error, GS_DOWNLOAD_ERROR, GS_DOWNLOAD_ERROR_NOT_MODIFIED)) {
+		gs_screenshot_image_stop_spinner (ssimg);
+		as_screenshot_show_image (ssimg);
+
+		g_clear_object (&ssimg->cancellable);
+	} else if (!g_error_matches (error, G_IO_ERROR, G_IO_ERROR_CANCELLED)) {
+		g_debug ("Failed to download screenshot video: %s", error->message);
+		/* Reset the width request, thus the image shrinks when the window width is small */
+		gtk_widget_set_size_request (ssimg->stack, -1, (gint) ssimg->height);
+		gs_screenshot_image_stop_spinner (ssimg);
+		gs_screenshot_image_set_error (ssimg, _("Screenshot not found"));
+	}
+}
+
 void
 gs_screenshot_image_load_async (GsScreenshotImage *ssimg,
 				GCancellable *cancellable)
 {
-	AsImage *im = NULL;
 	const gchar *url;
 	g_autofree gchar *basename = NULL;
 	g_autofree gchar *cache_kind = NULL;
 	g_autofree gchar *cachefn_thumb = NULL;
 	g_autofree gchar *sizedir = NULL;
-	g_autoptr(SoupURI) base_uri = NULL;
+	g_autoptr(GUri) base_uri = NULL;
 
 	g_return_if_fail (GS_IS_SCREENSHOT_IMAGE (ssimg));
 
@@ -485,20 +641,11 @@ gs_screenshot_image_load_async (GsScreenshotImage *ssimg,
 	g_return_if_fail (ssimg->width != 0);
 	g_return_if_fail (ssimg->height != 0);
 
-	/* load an image according to the scale factor */
-	ssimg->scale = (guint) gtk_widget_get_scale_factor (GTK_WIDGET (ssimg));
-	im = as_screenshot_get_image (ssimg->screenshot,
-				      ssimg->width * ssimg->scale,
-				      ssimg->height * ssimg->scale);
+	/* Reset the width request, thus the image shrinks when the window width is small */
+	gtk_widget_set_size_request (ssimg->stack, -1, (gint) ssimg->height);
 
-	/* if we've failed to load a HiDPI image, fallback to LoDPI */
-	if (im == NULL && ssimg->scale > 1) {
-		ssimg->scale = 1;
-		im = as_screenshot_get_image (ssimg->screenshot,
-					      ssimg->width,
-					      ssimg->height);
-	}
-	if (im == NULL) {
+	url = gs_screenshot_image_get_url (ssimg);
+	if (url == NULL) {
 		/* TRANSLATORS: this is when we request a screenshot size that
 		 * the generator did not create or the parser did not add */
 		gs_screenshot_image_set_error (ssimg, _("Screenshot size not found"));
@@ -506,7 +653,6 @@ gs_screenshot_image_load_async (GsScreenshotImage *ssimg,
 	}
 
 	/* check if the URL points to a local file */
-	url = as_image_get_url (im);
 	if (g_str_has_prefix (url, "file://")) {
 		g_free (ssimg->filename);
 		ssimg->filename = g_strdup (url + 7);
@@ -551,11 +697,13 @@ gs_screenshot_image_load_async (GsScreenshotImage *ssimg,
 	/* if we're not showing a full-size image, we try loading a blurred
 	 * smaller version of it straight away */
 	if (!ssimg->showing_image &&
+	    as_screenshot_get_media_kind (ssimg->screenshot) == AS_SCREENSHOT_MEDIA_KIND_IMAGE &&
 	    ssimg->width > AS_IMAGE_THUMBNAIL_WIDTH &&
 	    ssimg->height > AS_IMAGE_THUMBNAIL_HEIGHT) {
 		const gchar *url_thumb;
 		g_autofree gchar *basename_thumb = NULL;
 		g_autofree gchar *cache_kind_thumb = NULL;
+		AsImage *im;
 		im = as_screenshot_get_image (ssimg->screenshot,
 					      AS_IMAGE_THUMBNAIL_WIDTH * ssimg->scale,
 					      AS_IMAGE_THUMBNAIL_HEIGHT * ssimg->scale);
@@ -588,8 +736,12 @@ gs_screenshot_image_load_async (GsScreenshotImage *ssimg,
 
 	/* download file */
 	g_debug ("downloading %s to %s", url, ssimg->filename);
-	base_uri = soup_uri_new (url);
-	if (base_uri == NULL || !SOUP_URI_VALID_FOR_HTTP (base_uri)) {
+	base_uri = g_uri_parse (url, SOUP_HTTP_URI_FLAGS, NULL);
+	if (base_uri == NULL ||
+	    (g_strcmp0 (g_uri_get_scheme (base_uri), "http") != 0 &&
+	     g_strcmp0 (g_uri_get_scheme (base_uri), "https") != 0) ||
+	    g_uri_get_host (base_uri) == NULL ||
+	    g_uri_get_path (base_uri) == NULL) {
 		/* TRANSLATORS: this is when we try to download a screenshot
 		 * that was not a valid URL */
 		gs_screenshot_image_set_error (ssimg, _("Screenshot not valid"));
@@ -602,14 +754,44 @@ gs_screenshot_image_load_async (GsScreenshotImage *ssimg,
 	}
 
 	/* cancel any previous messages */
+	if (ssimg->cancellable != NULL) {
+		g_cancellable_cancel (ssimg->cancellable);
+		g_clear_object (&ssimg->cancellable);
+	}
+
 	if (ssimg->message != NULL) {
+#if !SOUP_CHECK_VERSION(3, 0, 0)
 		soup_session_cancel_message (ssimg->session,
 		                             ssimg->message,
 		                             SOUP_STATUS_CANCELLED);
+#endif
 		g_clear_object (&ssimg->message);
 	}
 
+	if (as_screenshot_get_media_kind (ssimg->screenshot) == AS_SCREENSHOT_MEDIA_KIND_VIDEO) {
+		g_autofree gchar *uri_str = g_uri_to_string (base_uri);
+		g_autoptr(GFile) output_file = NULL;
+
+		ssimg->cancellable = g_cancellable_new ();
+		output_file = g_file_new_for_path (ssimg->filename);
+
+		/* Make sure the spinner takes approximately the size the screenshot will use */
+		gtk_widget_set_size_request (ssimg->stack, (gint) ssimg->width, (gint) ssimg->height);
+
+		gs_download_file_async (ssimg->session, uri_str, output_file, G_PRIORITY_DEFAULT, NULL, NULL,
+					ssimg->cancellable, gs_screenshot_video_downloaded_cb, g_object_ref (ssimg));
+
+		return;
+	}
+
+#if SOUP_CHECK_VERSION(3, 0, 0)
 	ssimg->message = soup_message_new_from_uri (SOUP_METHOD_GET, base_uri);
+#else
+	{
+	g_autofree gchar *uri_str = g_uri_to_string (base_uri);
+	ssimg->message = soup_message_new (SOUP_METHOD_GET, uri_str);
+	}
+#endif
 	if (ssimg->message == NULL) {
 		/* TRANSLATORS: this is when networking is not available */
 		gs_screenshot_image_set_error (ssimg, _("Screenshot not available"));
@@ -627,10 +809,16 @@ gs_screenshot_image_load_async (GsScreenshotImage *ssimg,
 		gs_screenshot_show_spinner_cb, ssimg);
 
 	/* send async */
+#if SOUP_CHECK_VERSION(3, 0, 0)
+	ssimg->cancellable = g_cancellable_new ();
+	soup_session_send_async (ssimg->session, ssimg->message, G_PRIORITY_DEFAULT, ssimg->cancellable,
+				 gs_screenshot_image_complete_cb, g_object_ref (ssimg));
+#else
 	soup_session_queue_message (ssimg->session,
 				    g_object_ref (ssimg->message) /* transfer full */,
 				    gs_screenshot_image_complete_cb,
 				    g_object_ref (ssimg));
+#endif
 }
 
 gboolean
@@ -643,93 +831,115 @@ void
 gs_screenshot_image_set_description (GsScreenshotImage *ssimg,
 				     const gchar *description)
 {
-	AtkImage *atk_image;
-	atk_image = ATK_IMAGE (gtk_widget_get_accessible (ssimg->image1));
-	atk_image_set_image_description (atk_image, description);
-	atk_image = ATK_IMAGE (gtk_widget_get_accessible (ssimg->image2));
-	atk_image_set_image_description (atk_image, description);
+	gtk_accessible_update_property (GTK_ACCESSIBLE (ssimg->image1),
+					GTK_ACCESSIBLE_PROPERTY_DESCRIPTION, description,
+					-1);
+	gtk_accessible_update_property (GTK_ACCESSIBLE (ssimg->image2),
+					GTK_ACCESSIBLE_PROPERTY_DESCRIPTION, description,
+					-1);
 }
 
 static void
-gs_screenshot_image_destroy (GtkWidget *widget)
+gs_screenshot_image_dispose (GObject *object)
 {
-	GsScreenshotImage *ssimg = GS_SCREENSHOT_IMAGE (widget);
+	GsScreenshotImage *ssimg = GS_SCREENSHOT_IMAGE (object);
 
 	if (ssimg->load_timeout_id) {
 		g_source_remove (ssimg->load_timeout_id);
 		ssimg->load_timeout_id = 0;
 	}
 
+	if (ssimg->cancellable != NULL) {
+		g_cancellable_cancel (ssimg->cancellable);
+		g_clear_object (&ssimg->cancellable);
+	}
+
 	if (ssimg->message != NULL) {
+#if !SOUP_CHECK_VERSION(3, 0, 0)
 		soup_session_cancel_message (ssimg->session,
 		                             ssimg->message,
 		                             SOUP_STATUS_CANCELLED);
+#endif
 		g_clear_object (&ssimg->message);
 	}
+	gs_widget_remove_all (GTK_WIDGET (ssimg), NULL);
 	g_clear_object (&ssimg->screenshot);
 	g_clear_object (&ssimg->session);
 	g_clear_object (&ssimg->settings);
 
 	g_clear_pointer (&ssimg->filename, g_free);
 
-	GTK_WIDGET_CLASS (gs_screenshot_image_parent_class)->destroy (widget);
+	G_OBJECT_CLASS (gs_screenshot_image_parent_class)->dispose (object);
 }
 
 static void
 gs_screenshot_image_init (GsScreenshotImage *ssimg)
 {
-	AtkObject *accessible;
+	GtkGesture *gesture;
 
 	ssimg->settings = g_settings_new ("org.gnome.software");
 	ssimg->showing_image = FALSE;
 
-	gtk_widget_set_has_window (GTK_WIDGET (ssimg), FALSE);
-
-	g_type_ensure (GS_TYPE_PICTURE);
 	gtk_widget_init_template (GTK_WIDGET (ssimg));
 
-	accessible = gtk_widget_get_accessible (GTK_WIDGET (ssimg));
-	if (accessible != 0) {
-		atk_object_set_role (accessible, ATK_ROLE_IMAGE);
-		atk_object_set_name (accessible, _("Screenshot"));
-	}
+	gesture = gtk_gesture_click_new ();
+	g_signal_connect_object (gesture, "released",
+		G_CALLBACK (gs_screenshot_image_clicked_cb), ssimg, 0);
+	gtk_widget_add_controller (GTK_WIDGET (ssimg), GTK_EVENT_CONTROLLER (gesture));
 }
 
-static gboolean
-gs_screenshot_image_draw (GtkWidget *widget, cairo_t *cr)
+static void
+gs_screenshot_image_snapshot (GtkWidget   *widget,
+                              GtkSnapshot *snapshot)
 {
 	GtkStyleContext *context;
 
 	context = gtk_widget_get_style_context (widget);
-	gtk_render_background (context, cr,
-			       0, 0,
-			       gtk_widget_get_allocated_width (widget),
-			       gtk_widget_get_allocated_height (widget));
-	gtk_render_frame (context, cr,
-			  0, 0,
-			  gtk_widget_get_allocated_width (widget),
-			  gtk_widget_get_allocated_height (widget));
+	gtk_snapshot_render_frame (snapshot,
+				   context,
+				   0.0, 0.0,
+				   gtk_widget_get_width (widget),
+				   gtk_widget_get_height (widget));
 
-	return GTK_WIDGET_CLASS (gs_screenshot_image_parent_class)->draw (widget, cr);
+	GTK_WIDGET_CLASS (gs_screenshot_image_parent_class)->snapshot (widget, snapshot);
 }
 
 static void
 gs_screenshot_image_class_init (GsScreenshotImageClass *klass)
 {
+	GObjectClass *object_class = G_OBJECT_CLASS (klass);
 	GtkWidgetClass *widget_class = GTK_WIDGET_CLASS (klass);
 
-	widget_class->destroy = gs_screenshot_image_destroy;
-	widget_class->draw = gs_screenshot_image_draw;
+	object_class->dispose = gs_screenshot_image_dispose;
+
+	widget_class->snapshot = gs_screenshot_image_snapshot;
 
 	gtk_widget_class_set_template_from_resource (widget_class,
 						     "/org/gnome/Software/gs-screenshot-image.ui");
+	gtk_widget_class_set_layout_manager_type (widget_class, GTK_TYPE_BIN_LAYOUT);
+	gtk_widget_class_set_accessible_role (widget_class, GTK_ACCESSIBLE_ROLE_IMG);
 
 	gtk_widget_class_bind_template_child (widget_class, GsScreenshotImage, spinner);
 	gtk_widget_class_bind_template_child (widget_class, GsScreenshotImage, stack);
 	gtk_widget_class_bind_template_child (widget_class, GsScreenshotImage, image1);
 	gtk_widget_class_bind_template_child (widget_class, GsScreenshotImage, image2);
+	gtk_widget_class_bind_template_child (widget_class, GsScreenshotImage, video);
 	gtk_widget_class_bind_template_child (widget_class, GsScreenshotImage, box_error);
 	gtk_widget_class_bind_template_child (widget_class, GsScreenshotImage, label_error);
+
+	/**
+	 * GsScreenshotImage::clicked:
+	 *
+	 * Emitted when the screenshot is clicked.
+	 *
+	 * Since: 43
+	 */
+	signals [SIGNAL_CLICKED] =
+		g_signal_new ("clicked",
+			      G_TYPE_FROM_CLASS (object_class), G_SIGNAL_RUN_LAST,
+			      0,
+			      NULL, NULL, g_cclosure_marshal_VOID__VOID,
+			      G_TYPE_NONE, 0);
 }
 
 GtkWidget *
