@@ -19,7 +19,7 @@ G_BEGIN_DECLS
 
 #define GS_TYPE_SCREENSHOT_IMAGE (gs_screenshot_image_get_type ())
 
-G_DECLARE_FINAL_TYPE (GsScreenshotImage, gs_screenshot_image, GS, SCREENSHOT_IMAGE, GtkBin)
+G_DECLARE_FINAL_TYPE (GsScreenshotImage, gs_screenshot_image, GS, SCREENSHOT_IMAGE, GtkWidget)
 
 GtkWidget	*gs_screenshot_image_new		(SoupSession		*session);
 
@@ -32,5 +32,7 @@ void		 gs_screenshot_image_set_size		(GsScreenshotImage	*ssimg,
 void		 gs_screenshot_image_load_async		(GsScreenshotImage	*ssimg,
 							 GCancellable		*cancellable);
 gboolean	 gs_screenshot_image_is_showing		(GsScreenshotImage	*ssimg);
+void		 gs_screenshot_image_set_description	(GsScreenshotImage	*ssimg,
+							 const gchar		*description);
 
 G_END_DECLS
