@@ -20,18 +20,20 @@
 
 struct _GsUpdateDialog
 {
-	HdyWindow	 parent_instance;
+	AdwWindow	 parent_instance;
 
 	GCancellable	*cancellable;
 	GsPluginLoader	*plugin_loader;
 	GsApp		*app;
-	GtkWidget	*deck;
+	GtkWidget	*leaflet;
 	GtkWidget	*list_box_installed_updates;
 	GtkWidget	*spinner;
 	GtkWidget	*stack;
+	AdwWindowTitle	*window_title;
+	gboolean	 showing_installed_updates;
 };
 
-G_DEFINE_TYPE (GsUpdateDialog, gs_update_dialog, HDY_TYPE_WINDOW)
+G_DEFINE_TYPE (GsUpdateDialog, gs_update_dialog, ADW_TYPE_WINDOW)
 
 typedef enum {
 	PROP_PLUGIN_LOADER = 1,
@@ -44,26 +46,33 @@ static void gs_update_dialog_show_installed_updates (GsUpdateDialog *dialog);
 static void gs_update_dialog_show_update_details (GsUpdateDialog *dialog, GsApp *app);
 
 static void
-deck_child_transition_cb (HdyDeck *deck, GParamSpec *pspec, GsUpdateDialog *dialog)
+leaflet_child_transition_cb (AdwLeaflet *leaflet, GParamSpec *pspec, GsUpdateDialog *dialog)
 {
 	GtkWidget *child;
 
-	if (hdy_deck_get_transition_running (deck))
+	if (adw_leaflet_get_child_transition_running (leaflet))
 		return;
 
-	while ((child = hdy_deck_get_adjacent_child (deck, HDY_NAVIGATION_DIRECTION_FORWARD)))
-		gtk_widget_destroy (child);
+	while ((child = adw_leaflet_get_adjacent_child (leaflet, ADW_NAVIGATION_DIRECTION_FORWARD)))
+		adw_leaflet_remove (leaflet, child);
+
+	child = adw_leaflet_get_visible_child (leaflet);
+	if (child != NULL && g_object_class_find_property (G_OBJECT_CLASS (GTK_WIDGET_GET_CLASS (child)), "title") != NULL) {
+		g_autofree gchar *title = NULL;
+		g_object_get (G_OBJECT (child), "title", &title, NULL);
+		gtk_window_set_title (GTK_WINDOW (dialog), title);
+	} else if (dialog->showing_installed_updates) {
+		gtk_window_set_title (GTK_WINDOW (dialog), _("Installed Updates"));
+	} else {
+		gtk_window_set_title (GTK_WINDOW (dialog), "");
+	}
 }
 
 static void
-installed_updates_row_activated_cb (GtkListBox *list_box,
-				    GtkListBoxRow *row,
+installed_updates_row_activated_cb (GsUpdateList *update_list,
+				    GsApp *app,
 				    GsUpdateDialog *dialog)
 {
-	GsApp *app;
-
-	app = gs_app_row_get_app (GS_APP_ROW (row));
-
 	gs_update_dialog_show_update_details (dialog, app);
 }
 
@@ -83,12 +92,13 @@ get_installed_updates_cb (GsPluginLoader *plugin_loader,
 	/* if we're in teardown, short-circuit and return immediately without
 	 * dereferencing priv variables */
 	if (g_error_matches (error, GS_PLUGIN_ERROR, GS_PLUGIN_ERROR_CANCELLED) ||
+	    g_error_matches (error, G_IO_ERROR, G_IO_ERROR_CANCELLED) ||
 	    dialog->spinner == NULL) {
 		g_debug ("get installed updates cancelled");
 		return;
 	}
 
-	gs_stop_spinner (GTK_SPINNER (dialog->spinner));
+	gtk_spinner_stop (GTK_SPINNER (dialog->spinner));
 
 	/* error */
 	if (list == NULL) {
@@ -107,7 +117,6 @@ get_installed_updates_cb (GsPluginLoader *plugin_loader,
 	/* set the header title using any one of the applications */
 	install_date = gs_app_get_install_date (gs_app_list_index (list, 0));
 	if (install_date > 0) {
-		GtkWidget *header;
 		g_autoptr(GDateTime) date = NULL;
 		g_autofree gchar *date_str = NULL;
 		g_autofree gchar *subtitle = NULL;
@@ -120,13 +129,12 @@ get_installed_updates_cb (GsPluginLoader *plugin_loader,
 		   The date format is defined by the locale's preferred date representation
 		   ("%x" in strftime.) */
 		subtitle = g_strdup_printf (_("Installed on %s"), date_str);
-		header = gtk_dialog_get_header_bar (GTK_DIALOG (dialog));
-		gtk_header_bar_set_subtitle (GTK_HEADER_BAR (header), subtitle);
+		adw_window_title_set_subtitle (dialog->window_title, subtitle);
 	}
 
 	gtk_stack_set_visible_child_name (GTK_STACK (dialog->stack), "installed-updates-list");
 
-	gs_container_remove_all (GTK_CONTAINER (dialog->list_box_installed_updates));
+	gs_update_list_remove_all (GS_UPDATE_LIST (dialog->list_box_installed_updates));
 	for (i = 0; i < gs_app_list_length (list); i++) {
 		gs_update_list_add_app (GS_UPDATE_LIST (dialog->list_box_installed_updates),
 					gs_app_list_index (list, i));
@@ -138,10 +146,12 @@ gs_update_dialog_show_installed_updates (GsUpdateDialog *dialog)
 {
 	g_autoptr(GsPluginJob) plugin_job = NULL;
 
+	dialog->showing_installed_updates = TRUE;
+
 	/* TRANSLATORS: this is the title of the installed updates dialog window */
 	gtk_window_set_title (GTK_WINDOW (dialog), _("Installed Updates"));
 
-	gs_start_spinner (GTK_SPINNER (dialog->spinner));
+	gtk_spinner_start (GTK_SPINNER (dialog->spinner));
 	gtk_stack_set_visible_child_name (GTK_STACK (dialog->stack), "spinner");
 
 	plugin_job = gs_plugin_job_newv (GS_PLUGIN_ACTION_GET_UPDATES_HISTORICAL,
@@ -163,13 +173,12 @@ unset_focus (GtkWidget *widget)
 	focus = gtk_window_get_focus (GTK_WINDOW (widget));
 	if (GTK_IS_LABEL (focus))
 		gtk_label_select_region (GTK_LABEL (focus), 0, 0);
-	gtk_window_set_focus (GTK_WINDOW (widget), NULL);
 }
 
 static void
 back_clicked_cb (GtkWidget *widget, GsUpdateDialog *dialog)
 {
-	hdy_deck_navigate (HDY_DECK (dialog->deck), HDY_NAVIGATION_DIRECTION_BACK);
+	adw_leaflet_navigate (ADW_LEAFLET (dialog->leaflet), ADW_NAVIGATION_DIRECTION_BACK);
 }
 
 static void
@@ -201,58 +210,19 @@ gs_update_dialog_show_update_details (GsUpdateDialog *dialog, GsApp *app)
 		gs_os_update_page_set_app (GS_OS_UPDATE_PAGE (page), app);
 		g_signal_connect (page, "app-activated",
 				  G_CALLBACK (app_activated_cb), dialog);
+		gs_os_update_page_set_show_back_button (GS_OS_UPDATE_PAGE (page), dialog->showing_installed_updates);
 	} else {
 		page = gs_app_details_page_new ();
 		gs_app_details_page_set_app (GS_APP_DETAILS_PAGE (page), app);
-		g_signal_connect (page, "back-clicked",
-				  G_CALLBACK (back_clicked_cb), dialog);
 	}
 
+	g_signal_connect (page, "back-clicked",
+			  G_CALLBACK (back_clicked_cb), dialog);
+
 	gtk_widget_show (page);
 
-	gtk_container_add (GTK_CONTAINER (dialog->deck), page);
-	hdy_deck_set_visible_child (HDY_DECK (dialog->deck), page);
-}
-
-static gboolean
-key_press_event (GtkWidget *widget, GdkEventKey *event, gpointer user_data)
-{
-	GsUpdateDialog *dialog = (GsUpdateDialog *) widget;
-	GdkKeymap *keymap;
-	GdkModifierType state;
-	gboolean is_rtl;
-
-	state = event->state;
-	keymap = gdk_keymap_get_for_display (gtk_widget_get_display (widget));
-	gdk_keymap_add_virtual_modifiers (keymap, &state);
-	state = state & gtk_accelerator_get_default_mod_mask ();
-	is_rtl = gtk_widget_get_direction (GTK_WIDGET (dialog)) == GTK_TEXT_DIR_RTL;
-
-	if (event->keyval == GDK_KEY_Escape) {
-		gtk_window_close (GTK_WINDOW (dialog));
-
-		return GDK_EVENT_STOP;
-	}
-
-	if ((!is_rtl && state == GDK_MOD1_MASK && event->keyval == GDK_KEY_Left) ||
-	    (is_rtl && state == GDK_MOD1_MASK && event->keyval == GDK_KEY_Right) ||
-	    event->keyval == GDK_KEY_Back) {
-		hdy_deck_navigate (HDY_DECK (dialog->deck), HDY_NAVIGATION_DIRECTION_BACK);
-		return GDK_EVENT_STOP;
-	}
-
-	return GDK_EVENT_PROPAGATE;
-}
-
-static gboolean
-button_press_event (GsUpdateDialog *dialog, GdkEventButton *event)
-{
-	/* Mouse hardware back button is 8 */
-	if (event->button != 8)
-		return GDK_EVENT_PROPAGATE;
-
-	hdy_deck_navigate (HDY_DECK (dialog->deck), HDY_NAVIGATION_DIRECTION_BACK);
-	return GDK_EVENT_STOP;
+	adw_leaflet_append (ADW_LEAFLET (dialog->leaflet), page);
+	adw_leaflet_set_visible_child (ADW_LEAFLET (dialog->leaflet), page);
 }
 
 static void
@@ -289,7 +259,9 @@ gs_update_dialog_set_property (GObject      *object,
 		dialog->plugin_loader = g_object_ref (g_value_get_object (value));
 		break;
 	case PROP_APP:
-		dialog->app = g_object_ref (g_value_get_object (value));
+		dialog->app = g_value_get_object (value);
+		if (dialog->app != NULL)
+			g_object_ref (dialog->app);
 		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
@@ -307,12 +279,12 @@ gs_update_dialog_constructed (GObject *object)
 	if (dialog->app) {
 		GtkWidget *child;
 
-		child = hdy_deck_get_visible_child (HDY_DECK (dialog->deck));
-		gtk_container_remove (GTK_CONTAINER (dialog->deck), child);
+		child = adw_leaflet_get_visible_child (ADW_LEAFLET (dialog->leaflet));
+		adw_leaflet_remove (ADW_LEAFLET (dialog->leaflet), child);
 
 		gs_update_dialog_show_update_details (dialog, dialog->app);
 
-		child = hdy_deck_get_visible_child (HDY_DECK (dialog->deck));
+		child = adw_leaflet_get_visible_child (ADW_LEAFLET (dialog->leaflet));
 		/* It can be either the app details page or the OS update page */
 		if (GS_IS_APP_DETAILS_PAGE (child))
 			gs_app_details_page_set_show_back_button (GS_APP_DETAILS_PAGE (child), FALSE);
@@ -344,16 +316,10 @@ gs_update_dialog_init (GsUpdateDialog *dialog)
 
 	dialog->cancellable = g_cancellable_new ();
 
-	g_signal_connect (GTK_LIST_BOX (dialog->list_box_installed_updates), "row-activated",
+	g_signal_connect (dialog->list_box_installed_updates, "show-update",
 			  G_CALLBACK (installed_updates_row_activated_cb), dialog);
 
 	g_signal_connect_after (dialog, "show", G_CALLBACK (unset_focus), NULL);
-
-	/* global keynav and mouse back button */
-	g_signal_connect (dialog, "key-press-event",
-			  G_CALLBACK (key_press_event), NULL);
-	g_signal_connect (dialog, "button-press-event",
-			  G_CALLBACK (button_press_event), NULL);
 }
 
 static void
@@ -397,11 +363,14 @@ gs_update_dialog_class_init (GsUpdateDialogClass *klass)
 
 	gtk_widget_class_set_template_from_resource (widget_class, "/org/gnome/Software/gs-update-dialog.ui");
 
-	gtk_widget_class_bind_template_child (widget_class, GsUpdateDialog, deck);
+	gtk_widget_class_bind_template_child (widget_class, GsUpdateDialog, leaflet);
 	gtk_widget_class_bind_template_child (widget_class, GsUpdateDialog, list_box_installed_updates);
 	gtk_widget_class_bind_template_child (widget_class, GsUpdateDialog, spinner);
 	gtk_widget_class_bind_template_child (widget_class, GsUpdateDialog, stack);
-	gtk_widget_class_bind_template_callback (widget_class, deck_child_transition_cb);
+	gtk_widget_class_bind_template_child (widget_class, GsUpdateDialog, window_title);
+	gtk_widget_class_bind_template_callback (widget_class, leaflet_child_transition_cb);
+
+	gtk_widget_class_add_binding_action (widget_class, GDK_KEY_Escape, 0, "window.close", NULL);
 }
 
 GtkWidget *
