@@ -9,8 +9,8 @@
 
 #pragma once
 
+#include <adwaita.h>
 #include <gtk/gtk.h>
-#include <handy.h>
 
 #include "gnome-software-private.h"
 
@@ -18,7 +18,7 @@ G_BEGIN_DECLS
 
 #define GS_TYPE_UPDATE_DIALOG (gs_update_dialog_get_type ())
 
-G_DECLARE_FINAL_TYPE (GsUpdateDialog, gs_update_dialog, GS, UPDATE_DIALOG, HdyWindow)
+G_DECLARE_FINAL_TYPE (GsUpdateDialog, gs_update_dialog, GS, UPDATE_DIALOG, AdwWindow)
 
 GtkWidget	*gs_update_dialog_new				(GsPluginLoader		*plugin_loader);
 GtkWidget	*gs_update_dialog_new_for_app			(GsPluginLoader		*plugin_loader,
