@@ -53,12 +53,10 @@ struct _GsUpdatesPage
 	gboolean		 cache_valid;
 	guint			 action_cnt;
 	GsShell			*shell;
-	GsPluginStatus		 last_status;
 	GsUpdatesPageState	 state;
 	GsUpdatesPageFlags	 result_flags;
 	GtkWidget		*button_refresh;
 	GtkWidget		*header_spinner_start;
-	GtkWidget		*header_checking_label;
 	GtkWidget		*header_start_box;
 	gboolean		 has_agreed_to_mobile_data;
 	gboolean		 ampm_available;
@@ -74,12 +72,10 @@ struct _GsUpdatesPage
 	GtkWidget		*spinner_updates;
 	GtkWidget		*stack_updates;
 	GtkWidget		*upgrade_banner;
-	GtkWidget		*box_end_of_life;
+	GtkWidget		*infobar_end_of_life;
 	GtkWidget		*label_end_of_life;
 
-	GtkSizeGroup		*sizegroup_image;
 	GtkSizeGroup		*sizegroup_name;
-	GtkSizeGroup		*sizegroup_desc;
 	GtkSizeGroup		*sizegroup_button_label;
 	GtkSizeGroup		*sizegroup_button_image;
 	GtkSizeGroup		*sizegroup_header;
@@ -283,21 +279,12 @@ gs_updates_page_update_ui_state (GsUpdatesPage *self)
 	case GS_UPDATES_PAGE_STATE_STARTUP:
 	case GS_UPDATES_PAGE_STATE_ACTION_GET_UPDATES:
 	case GS_UPDATES_PAGE_STATE_ACTION_REFRESH:
-		/* if we have updates, avoid clearing the page with a spinner */
-		if (self->result_flags != GS_UPDATES_PAGE_FLAG_NONE) {
-			gs_stop_spinner (GTK_SPINNER (self->spinner_updates));
-			gtk_spinner_start (GTK_SPINNER (self->header_spinner_start));
-			gtk_widget_show (self->header_spinner_start);
-			gtk_widget_show (self->header_checking_label);
-		} else {
-			gs_start_spinner (GTK_SPINNER (self->spinner_updates));
-		}
+		gtk_spinner_start (GTK_SPINNER (self->spinner_updates));
 		break;
 	default:
-		gs_stop_spinner (GTK_SPINNER (self->spinner_updates));
+		gtk_spinner_stop (GTK_SPINNER (self->spinner_updates));
 		gtk_spinner_stop (GTK_SPINNER (self->header_spinner_start));
 		gtk_widget_hide (self->header_spinner_start);
-		gtk_widget_hide (self->header_checking_label);
 		break;
 	}
 
@@ -305,8 +292,7 @@ gs_updates_page_update_ui_state (GsUpdatesPage *self)
 	switch (self->state) {
 	case GS_UPDATES_PAGE_STATE_ACTION_REFRESH:
 	case GS_UPDATES_PAGE_STATE_ACTION_GET_UPDATES:
-		gtk_image_set_from_icon_name (GTK_IMAGE (gtk_button_get_image (GTK_BUTTON (self->button_refresh))),
-					      "media-playback-stop-symbolic", GTK_ICON_SIZE_MENU);
+		gtk_button_set_icon_name (GTK_BUTTON (self->button_refresh), "media-playback-stop-symbolic");
 		gtk_widget_show (self->button_refresh);
 		break;
 	case GS_UPDATES_PAGE_STATE_STARTUP:
@@ -314,8 +300,7 @@ gs_updates_page_update_ui_state (GsUpdatesPage *self)
 		gtk_widget_hide (self->button_refresh);
 		break;
 	case GS_UPDATES_PAGE_STATE_IDLE:
-		gtk_image_set_from_icon_name (GTK_IMAGE (gtk_button_get_image (GTK_BUTTON (self->button_refresh))),
-					      "view-refresh-symbolic", GTK_ICON_SIZE_MENU);
+		gtk_button_set_icon_name (GTK_BUTTON (self->button_refresh), "view-refresh-symbolic");
 		if (self->result_flags != GS_UPDATES_PAGE_FLAG_NONE) {
 			gtk_widget_show (self->button_refresh);
 		} else {
@@ -326,8 +311,7 @@ gs_updates_page_update_ui_state (GsUpdatesPage *self)
 		}
 		break;
 	case GS_UPDATES_PAGE_STATE_FAILED:
-		gtk_image_set_from_icon_name (GTK_IMAGE (gtk_button_get_image (GTK_BUTTON (self->button_refresh))),
-					      "view-refresh-symbolic", GTK_ICON_SIZE_MENU);
+		gtk_button_set_icon_name (GTK_BUTTON (self->button_refresh), "view-refresh-symbolic");
 		gtk_widget_show (self->button_refresh);
 		break;
 	default:
@@ -350,11 +334,7 @@ gs_updates_page_update_ui_state (GsUpdatesPage *self)
 						  "spinner");
 		break;
 	case GS_UPDATES_PAGE_STATE_ACTION_REFRESH:
-		if (self->result_flags != GS_UPDATES_PAGE_FLAG_NONE) {
-			gtk_stack_set_visible_child_name (GTK_STACK (self->stack_updates), "view");
-		} else {
-			gtk_stack_set_visible_child_name (GTK_STACK (self->stack_updates), "spinner");
-		}
+		gtk_stack_set_visible_child_name (GTK_STACK (self->stack_updates), "spinner");
 		break;
 	case GS_UPDATES_PAGE_STATE_STARTUP:
 	case GS_UPDATES_PAGE_STATE_IDLE:
@@ -451,9 +431,10 @@ gs_updates_page_get_updates_cb (GsPluginLoader *plugin_loader,
 	list = gs_plugin_loader_job_process_finish (plugin_loader, res, &error);
 	if (list == NULL) {
 		gs_updates_page_clear_flag (self, GS_UPDATES_PAGE_FLAG_HAS_UPDATES);
-		if (!g_error_matches (error, GS_PLUGIN_ERROR, GS_PLUGIN_ERROR_CANCELLED))
+		if (!g_error_matches (error, GS_PLUGIN_ERROR, GS_PLUGIN_ERROR_CANCELLED) &&
+		    !g_error_matches (error, G_IO_ERROR, G_IO_ERROR_CANCELLED))
 			g_warning ("updates-shell: failed to get updates: %s", error->message);
-		hdy_status_page_set_description (HDY_STATUS_PAGE (self->updates_failed_page),
+		adw_status_page_set_description (ADW_STATUS_PAGE (self->updates_failed_page),
 						 error->message);
 		gs_updates_page_set_state (self, GS_UPDATES_PAGE_STATE_FAILED);
 		refresh_headerbar_updates_counter (self);
@@ -496,7 +477,8 @@ gs_updates_page_get_upgrades_cb (GObject *source_object,
 	list = gs_plugin_loader_job_process_finish (plugin_loader, res, &error);
 	if (list == NULL) {
 		gs_updates_page_clear_flag (self, GS_UPDATES_PAGE_FLAG_HAS_UPGRADES);
-		if (!g_error_matches (error, GS_PLUGIN_ERROR, GS_PLUGIN_ERROR_CANCELLED)) {
+		if (!g_error_matches (error, GS_PLUGIN_ERROR, GS_PLUGIN_ERROR_CANCELLED) &&
+		    !g_error_matches (error, G_IO_ERROR, G_IO_ERROR_CANCELLED)) {
 			g_warning ("updates-shell: failed to get upgrades: %s",
 				   error->message);
 		}
@@ -558,14 +540,15 @@ gs_updates_page_refine_system_finished_cb (GObject *source_object,
 
 	/* get result */
 	if (!gs_plugin_loader_job_action_finish (plugin_loader, res, &error)) {
-		if (!g_error_matches (error, GS_PLUGIN_ERROR, GS_PLUGIN_ERROR_CANCELLED))
+		if (!g_error_matches (error, GS_PLUGIN_ERROR, GS_PLUGIN_ERROR_CANCELLED) &&
+		    !g_error_matches (error, G_IO_ERROR, G_IO_ERROR_CANCELLED))
 			g_warning ("Failed to refine system: %s", error->message);
 		return;
 	}
 
 	/* show or hide the end of life notification */
 	if (gs_app_get_state (app) != GS_APP_STATE_UNAVAILABLE) {
-		gtk_widget_set_visible (self->box_end_of_life, FALSE);
+		gtk_info_bar_set_revealed (GTK_INFO_BAR (self->infobar_end_of_life), FALSE);
 		return;
 	}
 
@@ -577,8 +560,7 @@ gs_updates_page_refine_system_finished_cb (GObject *source_object,
 					gs_app_get_name (app),
 					gs_app_get_version (app));
 	} else {
-		/* TRANSLATORS: OS refers to operating system, e.g. Fedora */
-		g_string_append (str, _("Your OS is no longer supported."));
+		g_string_append (str, _("Your operating system is no longer supported."));
 	}
 	g_string_append (str, " ");
 
@@ -590,7 +572,7 @@ gs_updates_page_refine_system_finished_cb (GObject *source_object,
 	g_string_append (str, _("It is recommended that you upgrade to a more recent version."));
 
 	gtk_label_set_label (GTK_LABEL (self->label_end_of_life), str->str);
-	gtk_widget_set_visible (self->box_end_of_life, TRUE);
+	gtk_info_bar_set_revealed (GTK_INFO_BAR (self->infobar_end_of_life), TRUE);
 
 }
 
@@ -609,7 +591,8 @@ gs_updates_page_get_system_finished_cb (GObject *source_object,
 
 	app = gs_plugin_loader_get_system_app_finish (plugin_loader, res, &error);
 	if (app == NULL) {
-		if (!g_error_matches (error, GS_PLUGIN_ERROR, GS_PLUGIN_ERROR_CANCELLED))
+		if (!g_error_matches (error, GS_PLUGIN_ERROR, GS_PLUGIN_ERROR_CANCELLED) &&
+		    !g_error_matches (error, G_IO_ERROR, G_IO_ERROR_CANCELLED))
 			g_warning ("Failed to get system: %s", error->message);
 		return;
 	}
@@ -622,11 +605,8 @@ gs_updates_page_get_system_finished_cb (GObject *source_object,
 		       GS_PLUGIN_REFINE_FLAGS_REQUIRE_VERSION;
 
 	helper = gs_page_helper_new (self, app);
-	plugin_job = gs_plugin_job_newv (GS_PLUGIN_ACTION_REFINE,
-					 "interactive", TRUE,
-					 "app", app,
-					 "refine-flags", refine_flags,
-					 NULL);
+	plugin_job = gs_plugin_job_refine_new_for_app (app, refine_flags);
+	gs_plugin_job_set_interactive (plugin_job, TRUE);
 	gs_plugin_loader_job_process_async (self->plugin_loader, plugin_job,
 					    self->cancellable,
 					    gs_updates_page_refine_system_finished_cb,
@@ -671,10 +651,8 @@ gs_updates_page_load (GsUpdatesPage *self)
 	if ((self->result_flags & GS_UPDATES_PAGE_FLAG_HAS_UPGRADES) == 0) {
 		refine_flags |= GS_PLUGIN_REFINE_FLAGS_REQUIRE_UPGRADE_REMOVED;
 		g_object_unref (plugin_job);
-		plugin_job = gs_plugin_job_newv (GS_PLUGIN_ACTION_GET_DISTRO_UPDATES,
-						 "interactive", TRUE,
-						 "refine-flags", refine_flags,
-						 NULL);
+		plugin_job = gs_plugin_job_list_distro_upgrades_new (GS_PLUGIN_LIST_DISTRO_UPGRADES_FLAGS_INTERACTIVE,
+								     refine_flags);
 		gs_plugin_loader_job_process_async (self->plugin_loader,
 						    plugin_job,
 						    self->cancellable,
@@ -744,14 +722,13 @@ gs_updates_page_refresh_cb (GsPluginLoader *plugin_loader,
 	ret = gs_plugin_loader_job_action_finish (plugin_loader, res, &error);
 	if (!ret) {
 		/* user cancel */
-		if (g_error_matches (error,
-				     GS_PLUGIN_ERROR,
-				     GS_PLUGIN_ERROR_CANCELLED)) {
+		if (g_error_matches (error, GS_PLUGIN_ERROR, GS_PLUGIN_ERROR_CANCELLED) ||
+		    g_error_matches (error, G_IO_ERROR, G_IO_ERROR_CANCELLED)) {
 			gs_updates_page_set_state (self, GS_UPDATES_PAGE_STATE_IDLE);
 			return;
 		}
 		g_warning ("failed to refresh: %s", error->message);
-		hdy_status_page_set_description (HDY_STATUS_PAGE (self->updates_failed_page),
+		adw_status_page_set_description (ADW_STATUS_PAGE (self->updates_failed_page),
 						 error->message);
 		gs_updates_page_set_state (self, GS_UPDATES_PAGE_STATE_FAILED);
 		return;
@@ -780,10 +757,8 @@ gs_updates_page_get_new_updates (GsUpdatesPage *self)
 	g_clear_object (&self->cancellable_refresh);
 	self->cancellable_refresh = g_cancellable_new ();
 
-	plugin_job = gs_plugin_job_newv (GS_PLUGIN_ACTION_REFRESH,
-					 "interactive", TRUE,
-					 "age", (guint64) 1,
-					 NULL);
+	plugin_job = gs_plugin_job_refresh_metadata_new (1,
+							 GS_PLUGIN_REFRESH_METADATA_FLAGS_INTERACTIVE);
 	gs_plugin_loader_job_process_async (self->plugin_loader, plugin_job,
 					    self->cancellable_refresh,
 					    (GAsyncReadyCallback) gs_updates_page_refresh_cb,
@@ -804,7 +779,7 @@ gs_updates_page_refresh_confirm_cb (GtkDialog *dialog,
                                     GsUpdatesPage *self)
 {
 	/* unmap the dialog */
-	gtk_widget_destroy (GTK_WIDGET (dialog));
+	gtk_window_destroy (GTK_WINDOW (dialog));
 
 	switch (response_type) {
 	case GTK_RESPONSE_REJECT:
@@ -929,7 +904,8 @@ upgrade_download_finished_cb (GObject *source,
 	g_autoptr(GError) error = NULL;
 
 	if (!gs_plugin_loader_job_action_finish (plugin_loader, res, &error)) {
-		if (g_error_matches (error, GS_PLUGIN_ERROR, GS_PLUGIN_ERROR_CANCELLED))
+		if (g_error_matches (error, GS_PLUGIN_ERROR, GS_PLUGIN_ERROR_CANCELLED) ||
+		    g_error_matches (error, G_IO_ERROR, G_IO_ERROR_CANCELLED))
 			return;
 		g_warning ("failed to upgrade-download: %s", error->message);
 	}
@@ -985,17 +961,15 @@ upgrade_reboot_failed_cb (GObject *source,
 	GsApp *app;
 	g_autoptr(GError) error = NULL;
 	g_autoptr(GsPluginJob) plugin_job = NULL;
-	g_autoptr(GVariant) retval = NULL;
 
 	/* get result */
-	retval = g_dbus_connection_call_finish (G_DBUS_CONNECTION (source), res, &error);
-	if (retval != NULL)
+	if (gs_utils_invoke_reboot_finish (source, res, &error))
 		return;
 
-	if (error != NULL) {
-		g_warning ("Calling org.gnome.SessionManager.Reboot failed: %s",
-			   error->message);
-	}
+	if (g_error_matches (error, G_IO_ERROR, G_IO_ERROR_CANCELLED))
+		g_debug ("Calling reboot had been cancelled");
+	else if (error != NULL)
+		g_warning ("Calling reboot failed: %s", error->message);
 
 	app = gs_upgrade_banner_get_app (GS_UPGRADE_BANNER (self->upgrade_banner));
 	if (app == NULL) {
@@ -1019,7 +993,6 @@ upgrade_trigger_finished_cb (GObject *source,
                              gpointer user_data)
 {
 	GsUpdatesPage *self = (GsUpdatesPage *) user_data;
-	g_autoptr(GDBusConnection) bus = NULL;
 	g_autoptr(GError) error = NULL;
 
 	/* get the results */
@@ -1029,16 +1002,7 @@ upgrade_trigger_finished_cb (GObject *source,
 	}
 
 	/* trigger reboot */
-	bus = g_bus_get_sync (G_BUS_TYPE_SESSION, NULL, NULL);
-	g_dbus_connection_call (bus,
-				"org.gnome.SessionManager",
-				"/org/gnome/SessionManager",
-				"org.gnome.SessionManager",
-				"Reboot",
-				NULL, NULL, G_DBUS_CALL_FLAGS_NONE,
-				G_MAXINT, NULL,
-				upgrade_reboot_failed_cb,
-				self);
+	gs_utils_invoke_reboot_async (NULL, upgrade_reboot_failed_cb, self);
 }
 
 static void
@@ -1069,7 +1033,7 @@ gs_updates_page_upgrade_confirm_cb (GtkDialog *dialog,
                                     GsUpdatesPage *self)
 {
 	/* unmap the dialog */
-	gtk_widget_destroy (GTK_WIDGET (dialog));
+	gtk_window_destroy (GTK_WINDOW (dialog));
 
 	switch (response_type) {
 	case GTK_RESPONSE_ACCEPT:
@@ -1197,7 +1161,6 @@ gs_updates_page_status_changed_cb (GsPluginLoader *plugin_loader,
 		break;
 	}
 
-	self->last_status = status;
 	gs_updates_page_update_ui_state (self);
 }
 
@@ -1229,17 +1192,13 @@ gs_updates_page_setup (GsPage *page,
                        GError **error)
 {
 	GsUpdatesPage *self = GS_UPDATES_PAGE (page);
-	AtkObject *accessible;
-	GtkWidget *widget;
 
 	g_return_val_if_fail (GS_IS_UPDATES_PAGE (self), TRUE);
 
 	for (guint i = 0; i < GS_UPDATES_SECTION_KIND_LAST; i++) {
 		self->sections[i] = gs_updates_section_new (i, plugin_loader, page);
 		gs_updates_section_set_size_groups (self->sections[i],
-						    self->sizegroup_image,
 						    self->sizegroup_name,
-						    self->sizegroup_desc,
 						    self->sizegroup_button_label,
 						    self->sizegroup_button_image,
 						    self->sizegroup_header);
@@ -1247,7 +1206,7 @@ gs_updates_page_setup (GsPage *page,
 		g_object_bind_property (G_OBJECT (self), "is-narrow",
 					self->sections[i], "is-narrow",
 					G_BINDING_SYNC_CREATE);
-		gtk_container_add (GTK_CONTAINER (self->updates_box), GTK_WIDGET (self->sections[i]));
+		gtk_box_append (GTK_BOX (self->updates_box), GTK_WIDGET (self->sections[i]));
 	}
 
 	self->shell = shell;
@@ -1282,36 +1241,15 @@ gs_updates_page_setup (GsPage *page,
 	gtk_widget_set_visible (self->header_start_box, TRUE);
 	gs_page_set_header_start_widget (GS_PAGE (self), self->header_start_box);
 
-	/* This label indicates that the update check is in progress */
-	self->header_checking_label = hdy_squeezer_new ();
-	hdy_squeezer_set_xalign (HDY_SQUEEZER (self->header_checking_label), 0);
-	hdy_squeezer_set_transition_type (HDY_SQUEEZER (self->header_checking_label), HDY_SQUEEZER_TRANSITION_TYPE_CROSSFADE);
-
-	widget = gtk_label_new (_("Checking…"));
-	gtk_widget_show (widget);
-	gtk_container_add (GTK_CONTAINER (self->header_checking_label), widget);
-
-	/* FIXME: This box is just a 0x0 widget the squeezer will show when it
-	 * hasn't enough space to show the label. In GTK 4, we will be able to
-	 * use AdwSqueezer:allow-none instead. */
-	widget = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 0);
-	gtk_widget_show (widget);
-	gtk_container_add (GTK_CONTAINER (self->header_checking_label), widget);
-
-	gtk_container_add (GTK_CONTAINER (self->header_start_box), self->header_checking_label);
-	gtk_container_child_set(GTK_CONTAINER (self->header_start_box), self->header_checking_label,
-				"pack-type", GTK_PACK_END, NULL);
 	self->header_spinner_start = gtk_spinner_new ();
-	gtk_container_add (GTK_CONTAINER (self->header_start_box), self->header_spinner_start);
-	gtk_container_child_set (GTK_CONTAINER (self->header_start_box), self->header_spinner_start,
-				 "pack-type", GTK_PACK_END, NULL);
+	gtk_box_prepend (GTK_BOX (self->header_start_box), self->header_spinner_start);
 
 	/* setup update details window */
-	self->button_refresh = gtk_button_new_from_icon_name ("view-refresh-symbolic", GTK_ICON_SIZE_MENU);
-	accessible = gtk_widget_get_accessible (self->button_refresh);
-	if (accessible != NULL)
-		atk_object_set_name (accessible, _("Check for updates"));
-	gtk_container_add (GTK_CONTAINER (self->header_start_box), self->button_refresh);
+	self->button_refresh = gtk_button_new_from_icon_name ("view-refresh-symbolic");
+	gtk_accessible_update_property (GTK_ACCESSIBLE (self->button_refresh),
+					GTK_ACCESSIBLE_PROPERTY_LABEL, _("Check for updates"),
+					-1);
+	gtk_box_prepend (GTK_BOX (self->header_start_box), self->button_refresh);
 	g_signal_connect (self->button_refresh, "clicked",
 			  G_CALLBACK (gs_updates_page_button_refresh_cb),
 			  self);
@@ -1323,14 +1261,6 @@ gs_updates_page_setup (GsPage *page,
 			  G_CALLBACK (gs_updates_page_button_network_settings_cb),
 			  self);
 
-	/* visually aligned */
-	self->sizegroup_image = gtk_size_group_new (GTK_SIZE_GROUP_HORIZONTAL);
-	self->sizegroup_name = gtk_size_group_new (GTK_SIZE_GROUP_HORIZONTAL);
-	self->sizegroup_desc = gtk_size_group_new (GTK_SIZE_GROUP_HORIZONTAL);
-	self->sizegroup_button_label = gtk_size_group_new (GTK_SIZE_GROUP_HORIZONTAL);
-	self->sizegroup_button_image = gtk_size_group_new (GTK_SIZE_GROUP_HORIZONTAL);
-	self->sizegroup_header = gtk_size_group_new (GTK_SIZE_GROUP_VERTICAL);
-
 	/* set initial state */
 	if (!gs_plugin_loader_get_allow_updates (self->plugin_loader))
 		self->state = GS_UPDATES_PAGE_STATE_MANAGED;
@@ -1353,7 +1283,7 @@ gs_updates_page_get_property (GObject    *object,
 		g_value_set_object (value, gtk_scrolled_window_get_vadjustment (GTK_SCROLLED_WINDOW (self->scrolledwindow_updates)));
 		break;
 	case PROP_TITLE:
-		g_value_set_string (value, _("Updates"));
+		g_value_set_string (value, C_("Apps to be updated", "Updates"));
 		break;
 	case PROP_COUNTER:
 		g_value_set_uint (value, self->updates_counter);
@@ -1402,7 +1332,7 @@ gs_updates_page_dispose (GObject *object)
 
 	for (guint i = 0; i < GS_UPDATES_SECTION_KIND_LAST; i++) {
 		if (self->sections[i] != NULL) {
-			gtk_widget_destroy (GTK_WIDGET (self->sections[i]));
+			gtk_widget_unparent (GTK_WIDGET (self->sections[i]));
 			self->sections[i] = NULL;
 		}
 	}
@@ -1412,9 +1342,7 @@ gs_updates_page_dispose (GObject *object)
 	g_clear_object (&self->settings);
 	g_clear_object (&self->desktop_settings);
 
-	g_clear_object (&self->sizegroup_image);
 	g_clear_object (&self->sizegroup_name);
-	g_clear_object (&self->sizegroup_desc);
 	g_clear_object (&self->sizegroup_button_label);
 	g_clear_object (&self->sizegroup_button_image);
 	g_clear_object (&self->sizegroup_header);
@@ -1471,7 +1399,7 @@ gs_updates_page_class_init (GsUpdatesPageClass *klass)
 	gtk_widget_class_bind_template_child (widget_class, GsUpdatesPage, spinner_updates);
 	gtk_widget_class_bind_template_child (widget_class, GsUpdatesPage, stack_updates);
 	gtk_widget_class_bind_template_child (widget_class, GsUpdatesPage, upgrade_banner);
-	gtk_widget_class_bind_template_child (widget_class, GsUpdatesPage, box_end_of_life);
+	gtk_widget_class_bind_template_child (widget_class, GsUpdatesPage, infobar_end_of_life);
 	gtk_widget_class_bind_template_child (widget_class, GsUpdatesPage, label_end_of_life);
 }
 
@@ -1484,9 +1412,7 @@ gs_updates_page_init (GsUpdatesPage *self)
 	self->settings = g_settings_new ("org.gnome.software");
 	self->desktop_settings = g_settings_new ("org.gnome.desktop.interface");
 
-	self->sizegroup_image = gtk_size_group_new (GTK_SIZE_GROUP_HORIZONTAL);
 	self->sizegroup_name = gtk_size_group_new (GTK_SIZE_GROUP_HORIZONTAL);
-	self->sizegroup_desc = gtk_size_group_new (GTK_SIZE_GROUP_HORIZONTAL);
 	self->sizegroup_button_label = gtk_size_group_new (GTK_SIZE_GROUP_HORIZONTAL);
 	self->sizegroup_button_image = gtk_size_group_new (GTK_SIZE_GROUP_HORIZONTAL);
 	self->sizegroup_header = gtk_size_group_new (GTK_SIZE_GROUP_VERTICAL);
